/*
* database-sync.js
*/

/*
* CONTENTS
*
* check for data
* check if dropbox data newer
* save database
*    fn: doSave
*    backup all tables
*    serialize data
*    save to dropbox file
*    fn: backup a table to json
*    fn: save serialized db to dropbox
* restore database
*    open file from dropbox
*    fn: confirmRestore
*    fn: doRestore
*    get link to db
*    delete all of the table rows
*    restore all of the tables
*    fn: delete table
*    fn: restore table
* get revisions
*    fn: format revision date nicely
* restore revision
* generic functions
*    fn: generate sync id
*/
		
	
	/*
		> check for data
		  see if there is already data on Dropbox first time user logs in
	*/

		balance.webdb.checkForData = function(callback) {

			console.log('checkForData');

			// arguments
            var args = JSON.stringify({
            	'path': '/Fudget.data'
			});

            $.ajax({
                url: 'https://api.dropboxapi.com/2/files/get_metadata',
                type: 'POST',
                beforeSend: function (xhr) {
                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
                    xhr.setRequestHeader('Content-Type', 'application/json');
                },
                data: args,
                // data found on dropbox
                success: function( response ) {
                	console.log('Data found on Dropbox');

                	// confirm download dropbox data
		       		nativeConfirm(
		       			'Download the Dropbox data to this device? \n\r Note: this will replace the data already on this device',	// message
		       			downloadDropboxDataConfirmCallback,					                                                        // callback
		       			'Data found on Dropbox',						                                                            // title
		       			['Yes, download Dropbox data', 'No, use data from this device'],							                // button labels
		       			null,									                                                                    // fallback message
		       			1,										                                                                    // fallback - ok pressed
		       			2										                                                                    // fallback - cancel pressed
		       		);

		       		// confirm download dropbox data callback
		       		function downloadDropboxDataConfirmCallback(buttonIndex) {
		       			// yes pressed
		       			if (buttonIndex === 1) {
		       				balance.webdb.restoreDatabase(balance.webdb.getAllBudgets);	
		       			}
		       			// no pressed
		       			else if (buttonIndex === 2) {
		       				balance.webdb.saveDatabase();
		       			}
		       		}
	       			
                },
                // no data found on dropbox
				error: function(XMLHttpRequest, textStatus, errorThrown) {
					var responseText = XMLHttpRequest.responseText;
					// if no file found, save the user's current data to dropbox and start checking for changes
					if (responseText.indexOf('not_found') > -1) {
						console.log('No data on dropbox');
						// startCheckingForChanges();
					}
				}
            });

		}


	/*
		> check if dropbox data newer
	*/

		balance.webdb.checkIfDropboxDataNewer = function(callback) {

			balance.webdb.restoreDatabase(balance.webdb.getAllBudgets, true);

		}


	/*
		> save database
	*/
	
		var data = {};

		balance.webdb.saveDatabase = function(callback) {

			// only if dropbox save & restore enabled
			if (localStorage.dropboxSaveAndRestore === 'true') {

				// confirm save
        		nativeConfirm(
        			'Save data to Dropbox? (This will overwrite data on Dropbox)',	// message
        			confirmSaveCallback,					                        // callback
        			'Save to Dropbox?',						                        // title
        			['No', 'Yes'],							                        // button labels
        			null,									                        // fallback message
        			2										                        // fallback - ok pressed
        		);

        		// confirm save callback
        		function confirmSaveCallback(buttonIndex) {
        			// yes pressed
    		     	if (buttonIndex === 2) {
    		     		doSave(callback);
    		     	}
        		}

			}

		}

		// >> fn: doSave
		function doSave(callback) {

			data = {};

			showLoadingScreen('Saving to Dropbox...');

			// >> backup all tables
			$.when(
			    backup('budgets'), 
			    backup('entries'),
			    backup('budgets_entries'),
			    backup('paid'),
			    backup('dates')
			).then(function(jsonData) {
				// console.log(jsonData);
				// add a random sync id to the json
				var newSyncId = generateSyncId();
				jsonData['syncId'] = newSyncId;
				// add the sync date to the json (milliseconds since epoch)
				var newSyncDate = new Date().getTime();
				jsonData['syncDate'] = newSyncDate;
			    // >> serialize data
			    var serializedData = JSON.stringify(jsonData);
			    // >> save to dropbox file
			    saveSerializedDbToDropbox(serializedData, newSyncId, newSyncDate, callback);
			});

		}

		// >> fn: backup a table to json
		function backup(table) {
			var db = balance.webdb.db;
		    var def = new $.Deferred();
		    data[table] = [];
		    db.transaction(function(tx) {
		    	// select everything from table
		        tx.executeSql("select * from "+table, [], function(tx,results) {
		        	for (var i=0; i < results.rows.length; i++) {
		        		// loop through each key in the row and push only the value to an array
		        		var rowArray = [];
		        		for (var key in results.rows.item(i)) {
		        		  rowArray.push(results.rows.item(i)[key]);
		        		}
		        		// push the array of only values to the data object
		        		data[table].push(rowArray);
		        	}
		            // var data = results;
		            def.resolve(data);
		        });
		    }, balance.webdb.onError);

		    return def;
		}

		// >> fn: save serialized db to dropbox
		function saveSerializedDbToDropbox(serializedData, newSyncId, newSyncDate, callback) {

			// if zipping is enabled, zip the data first
			if (dropboxZipMode) {
				console.log('dropboxZipMode enabled');
				// zip the serialized data
				var compression_mode = 9,
				    zip = LZMA; /// lzma_worker.js creates a global LZMA object.

			    /// First, let's compress it.
			    zip.compress(serializedData, compression_mode, function on_compress_complete(result) {

			    	var data = JSON.stringify(result);
			    	console.log(data);
			    	uploadToDropbox(data);

			    }, null);
			}
			// else don't zip the data
			else {
				uploadToDropbox(serializedData);
			}

		    // fn: upload to dropbox
		    function uploadToDropbox(result) {
		    	console.log('result', result);
				// arguments
	            var args = JSON.stringify({
	            	'path': '/Fudget.data',
				    'mode': 'overwrite', // add, overwrite, update
				    'autorename': false,
				    'mute': false
				});
				// upload to dropbox
	            $.ajax({
	                url: 'https://content.dropboxapi.com/2/files/upload',
	                type: 'POST',
	                beforeSend: function (xhr) {
	                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
	                    xhr.setRequestHeader('Content-Type', 'application/octet-stream');
	                    xhr.setRequestHeader('Dropbox-API-Arg', args);
	                },
	                data: result,
	                success: function( response ) {
	                    // alert('response' + response);
	                    log('Database saved to Dropbox!');
	                    console.log('Database saved to Dropbox!');
	                    hideLoadingScreen();
						showNotification('Data saved to Dropbox', 'success', 2000);
	                    // save the syncId to localStorage
	                    localStorage.syncId = newSyncId;
	                    // save the syncDate to localStorage
	                    localStorage.syncDate = newSyncDate;
	                    // start checking for changes locallly
	                    // startCheckingForChangesLocally();
	                    if (callback) {
	                    	callback();
	                    }

	                },
					error: function(XMLHttpRequest, textStatus, errorThrown) {
						console.dir(XMLHttpRequest);
						console.log(XMLHttpRequest.responseText);
						hideLoadingScreen();

						// notify user we couldn't save to Dropbox
			       		nativeAlert(
			       			'Could not save to Dropbox. Please check your internet and try again.',
			       			null,
			       			'Couldn\'t save to Dropbox'
			       		);
					}
	            });				    	
		    }

		}


	/*
		> restore database
	*/

		balance.webdb.restoreDatabase = function(callback, startup, skipConfirm) {

			// only if dropbox save & restore enabled
			if (localStorage.dropboxSaveAndRestore === 'true') {

				console.log('Start getting data from Dropbox');

				showLoadingScreen('Connecting to Dropbox...');

				// arguments
	            var args = JSON.stringify({
	            	'path': '/Fudget.data'
				});
				// >> open file from dropbox
	            $.ajax({
	                url: 'https://content.dropboxapi.com/2/files/download',
	                type: 'POST',
	                beforeSend: function (xhr) {
	                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
	                    xhr.setRequestHeader('Dropbox-API-Arg', args);
	                },
	                success: function( response ) {

	                	// if zipping is enabled, unzip the data first
	                	if (dropboxZipMode) {

		                	var zipArray = JSON.parse(response);

		                	/// unzip the zipped code
		                	zip = LZMA; /// lzma_worker.js creates a global LZMA object.
		                	zip.decompress(zipArray, function on_decompress_complete(result) {

		                		confirmRestore(result);

		                	}, null);

	                	}
	                	// else don't unzip the data
	                	else {
	                		confirmRestore(response);
	                	}

	                	// >> fn: confirmRestore
	                	function confirmRestore(data) {

	                		// if checking for new data on app startup
	                		if (startup) {

	                			// get json from data string
	                			var jsonData = JSON.parse(data);

	                			// get dropbox sync id
	                			var syncId = jsonData.syncId;

	                			// if dropbox sync id does not match local sync id
	                			if (syncId !== localStorage.syncId) {
	                				console.log('Dropbox data is newer');
	                				showConfirm('New data found on Dropbox. Restore and overwrite data on this device?');
	                			}
	                			// else data on dropbox is same, hide loading screen
	                			else {
	                				hideLoadingScreen();
	                			}
	                		}

	                		else {
	                			showConfirm('Restore data from Dropbox? (This will overwrite data on this device)');
	                		}

	                		function showConfirm(message) {

		                		hideLoadingScreen();

		                		// if we're skipping confirm, skip it
		                		if (skipConfirm) {
		                			doRestore(data);
		                		}
		                		// otherwise show the confirm
		                		else {

			                		// confirm restore data
			                		nativeConfirm(
			                			message,	                // message
			                			confirmRestoreCallback,	    // callback
			                			'Restore from Dropbox?',	// title
			                			['No', 'Yes'],	            // button labels
			                			null,	                    // fallback message
			                			2	                        // fallback - ok pressed
			                		);

			                		// confirm restore data callback
			                		function confirmRestoreCallback(buttonIndex) {
			                			// yes pressed
										if (buttonIndex === 2) {
											doRestore(data);
										}
			                		}

			                	}

	                		}

	                	} 

	                	// >> fn: doRestore
	                	function doRestore(data) {

	                		showLoadingScreen('Restoring from Dropbox...');

		                    var jsonString = data;
		                    var jsonData = JSON.parse(jsonString);

                   			// >> get link to db
                   			var db = balance.webdb.db;

                   		    // >> delete all of the table rows
                   		    $.when(
                   		        deleteTable('budgets', db), 
                   		        deleteTable('entries', db),
                   		        deleteTable('budgets_entries', db),
                   		        deleteTable('paid', db),
                   		        deleteTable('dates', db)
                   		    ).then(function() {
                   		    	// >> restore all of the tables
                   		    	$.when(
                   		    		restoreTable('budgets', db, jsonData),
                   		    		restoreTable('entries', db, jsonData),
                   		    		restoreTable('budgets_entries', db, jsonData),
                   		    		restoreTable('paid', db, jsonData),
                   		    		restoreTable('dates', db, jsonData)
                   		    	).then(function(){
                   		    		// restore syncId
                   		    		localStorage.syncId = jsonData['syncId'];
                   		    		// restore syncDate
                   		    		localStorage.syncDate = jsonData['syncDate'];
                   		    		// flag to show if user has restored a database 
                   		    		// at least once
                   		    		localStorage.previouslyRestored = true;
                   		    		log('Database restored!');
                   		    		console.log('Database restored!');

                   		    		hideLoadingScreen();

                   		    		// show notification
                   		    		showNotification('Data restored from Dropbox!', 'success', 3000);
                   		    		// fire the callback if there is one
                   		    		if (callback) {
                   		    			callback();
                   		    		}
                   		    	});

                   		    });
	                	}


	                },
					error: function(XMLHttpRequest, textStatus, errorThrown) {

						console.log('could not restore');

						hideLoadingScreen();

						console.dir(XMLHttpRequest);
						console.log(XMLHttpRequest.responseText);

						// notify user we couldn't restore from Dropbox
			       		nativeAlert(
			       			'Could not restore from Dropbox. Please check your internet and try again.',
			       			null,
			       			'Couldn\'t restore from Dropbox'
			       		);

			       		// fire callback if there is one
						if (callback) {
							callback();
						}
					}
	            });		

			}

			// else go straight to the callback (if there is one)
			else if (callback) {
				callback();
			}

		}

		// >> fn: delete table
		function deleteTable(table, db) {
			var def = new $.Deferred();
		    db.transaction(function(tx) {
		        tx.executeSql("DELETE FROM " + table, [], function(tx,results) {
		            def.resolve(results);
		        });
		    }, function() {console.log('delete table error');});

		    return def;	
		}

		// >> fn: restore table
		function restoreTable(table, db, jsonData) {
			db.transaction(function(tx) {
        		// build up the question marks part of the query based on the number of columns e.g. ?, ?, ?
        		q = '';
        		for (var j in jsonData[table][0]) {
        			q += (q == "" ? "" : ", ") + "?";
        		}
        		
        		// for each row of this table
        		for (var i=0; i < jsonData[table].length; i++) {
        			var query = 'INSERT OR IGNORE INTO ' + table + ' VALUES (' + q + ')';
					tx.executeSql(query, jsonData[table][i], null, balance.webdb.onError);
				}

			});
		}


	/*
		> get revisions
	*/

		balance.webdb.getRevisions = function() {

			console.log('getRevisions');

			showLoadingScreen('Getting revision history...');

			// arguments
            var args = JSON.stringify({
            	'path': '/Fudget.data',
            	'limit': 40
			});

            $.ajax({
                url: 'https://api.dropboxapi.com/2/files/list_revisions',
                type: 'POST',
                beforeSend: function (xhr) {
                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
                    xhr.setRequestHeader('Content-Type', 'application/json');
                },
                data: args,
                // revisions found
                success: function( response ) {
                	console.log('response: ', response);

                	// get revision entries
                	var revisionEntries = response.entries;

                	// populate dropdown
                	var revisionEntriesMarkup = '<option value="">Choose a revision to restore</option>';
                	for (i = 0; i < revisionEntries.length; i++) {
	                	// add tick to first item (to show current revision)
	                	if (i === 0 && !response.is_deleted) {
	                		var tick = ' &#10004;';
	                	}
	                	else {
	                		var tick = '';
	                	}
                		revisionEntriesMarkup += '<option value="' + revisionEntries[i].rev + '">' + niceRevisionDate(revisionEntries[i].client_modified) + tick + '</option>';
                	}
                	$('#choose-revision').html(revisionEntriesMarkup);

                	// show restore from revision item
                	$('.restore-data-from-revision').removeClass('hidden');

                	gettingRevisions = false;

                	// hide loading screen
                	hideLoadingScreen();
                },
                // no revisions found
				error: function(XMLHttpRequest, textStatus, errorThrown) {
					var responseText = XMLHttpRequest.responseText;
					console.log('responseText: ', responseText);
				}
            });			
		}

		// >> fn: format revision date nicely
		function niceRevisionDate(dateString, count) {
			// get date object
			var formattedDate = new Date(dateString);

			// get all the variables we need
			var date = formattedDate.getDate();
			var dayNames = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
			var day = formattedDate.getDay();
			var monthNames = [
			  'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
			];
			var month =  formattedDate.getMonth();
			var year = formattedDate.getFullYear();
			var hour = formattedDate.getHours();
			var minute = formattedDate.getMinutes();
			var seconds = formattedDate.getSeconds();

			// add zero padding
			if (hour < 10) {
				hour = '0' + hour;
			}
			if (minute < 10) {
				minute = '0' + minute;
			}
			if (seconds < 10) {
				seconds = '0' + seconds;
			}

			// create the full string
			var outputDate = dayNames[day] + ', ' + monthNames[month] + ' ' + date + ' @ ' + hour + ':' + minute + ':' + seconds;

			return outputDate; 
		}


	/*
		> restore revision
	*/

		balance.webdb.restoreRevision = function(rev) {

			showLoadingScreen('Restoring revision...');

			// arguments
            var args = JSON.stringify({
            	'path': '/Fudget.data',
            	'rev': rev
			});

            $.ajax({
                url: 'https://api.dropboxapi.com/2/files/restore',
                type: 'POST',
                beforeSend: function (xhr) {
                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
                    xhr.setRequestHeader('Content-Type', 'application/json');
                },
                data: args,
                // revisions found
                success: function( response ) {
                	console.log('response: ', response);

                	// hide loading screen
                	hideLoadingScreen();

                	// hide 'restore from revision' option
                	$('.restore-data-from-revision').addClass('hidden');

                	// file is restored, now restore data from that file
                	balance.webdb.restoreDatabase(balance.webdb.getAllBudgets, false, true);

                },
                // couldn't restore
				error: function(XMLHttpRequest, textStatus, errorThrown) {
					var responseText = XMLHttpRequest.responseText;
					shout('Could not restore revision: ' + responseText);
				}
            });			

		}


	/*
		> generic functions
	*/

		// >> fn: generate sync id
		function generateSyncId() {
		    var text = "";
		    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

		    for( var i=0; i < 15; i++ )
		        text += possible.charAt(Math.floor(Math.random() * possible.length));

		    return text;
		}