/*
* global.js
*/

/*
* CONTENTS
*
* window load
*    native or browser?
* onDeviceReady
*    device detection
*    google analytics
*    version number check
*    setup pro version (if purchased) & check if pro trial expired
*    fastclick - makes taps fire quicker on mobile
*    initial login
*    database setup
*    global variables
*    instructions
*    settings - switches
*    settings - set or change currency
*    settings - pin code lock
*    settings - touch id login
*    settings - email address
*    settings - theme submenu
*      show theme submenu
*      back to settings from theme submenu
*      change theme
*      set theme on page load
*    settings - click 'tell a friend'
*    settings - click rate this app
*    settings - click email me
*    settings - click more apps
*    settings - books
*    settings - click clear data
*    touch feedback
*    initializations
*    sortable
*    show budget list on swipe right
*    show settings on swipe right
*    show budgets on swipe left
*    tap budget - show entry list
*    autogrow textareas
*    pro upgrade banner
*    tap or blur entry title
*    tap or blur entry amount
*    blur entry (update)
*    change entry date
*    calculator
*    calculate total income
*    tap add entry
*    swipe left - show options
*    tap delete entry
*    mark as paid & unpaid
*    star or unstar entry
*    tap add budget
*    tap delete budget
*    tap rename budget
*    tap export budget
*    blur budget (update)
*    toggle total income or expenses
*    toggle balance or paid
*    disable unwanted key presses
*    tutorial
*      show or hide tutorial
*    popups
*      pro upgrade popup
*    in-app purchases
*    tap title bar - scroll to top
*    show options menu
*    join newsletter
*    charts
*    toggle entry date column
*    tap show review icon
*    android back button
*    dropbox save & restore setup
*    check data on dropbox
*    dropbox save database
*    dropbox restore database
*    persistent storage - restore
*    persistent storage - stop asking
*    persistent storage - save
*    persistent storage - check if database deleted by ios
* onAppPaused
* onAppResumed
*/


/* -----------------------------------------------------------------------------------
	> window load
--------------------------------------------------------------------------------------- */

	$(window).load(function(){

	    /*
	    	>> native or browser?
	    */
	    
		    // isNative = (document.URL.indexOf("http://") === -1 && document.URL.indexOf("https://") === -1);

		    // isNative = (document.URL.indexOf('file:///private') > -1)

		    if (window.cordova) {
		    	isNative = true;
		    }
		    else {
		    	isNative = false;
		    }

		    if( isNative ) {
		        document.addEventListener("deviceready", onDeviceReady, false);
		        document.addEventListener("pause", onAppPaused, false);
		        document.addEventListener("resume", onAppResumed, false);
		    } else {  
		        onDeviceReady();
		    }

			
	});

/* -----------------------------------------------------------------------------------
	> onDeviceReady
--------------------------------------------------------------------------------------- */

	function onDeviceReady() {

		/*
			>> device detection
		*/

			if (isNative) {
				platform = device.platform; // 'iOS' / 'Android'
				if (platform === 'iOS') {
					$('body').addClass('ios');

					// variables 
					appReviewUrl = 'http://itunes.apple.com/WebObjects/MZStore.woa/wa/viewContentsUserReviews?pageNumber=0&sortOrdering=1&type=Purple+Software&mt=8&id=897849266';
					moreAppsUrl = 'itms-apps://itunes.com/apps/dannyconnell/';
					feedbackUrl = 'mailto:fudget@dconnell.co.uk?subject=Fudget - iOS - Feedback&body=' + getIosDeviceInfoForEmail();

					function getIosDeviceInfoForEmail() {

						if (window.cordova) {
							try {
								var deviceInfo = 	'\n\n';
									deviceInfo +=	'-- Please write message above this line --'
									deviceInfo +=	'\n\n';
									deviceInfo +=	'Platform: ' + device.platform + '\n';
									deviceInfo +=	'Platform version: ' + device.version + '\n';	
									deviceInfo +=	'Device model: ' + deviceModels[device.model] + ' (' + device.model + ') ' + '\n';	
									deviceInfo +=	'Fudget version: ' + versionNumber;	

								var deviceInfoEncoded = encodeURI(deviceInfo);
							}
							catch(error) {
								var deviceInfoEncoded = '';
							}
						}
						else {
							deviceInfoEncoded = '';
						}
						
						return deviceInfoEncoded;
					}
				}
				if (platform === 'Android') {
					$('body').addClass('android');

					// individual versions
					var androidVersion = parseFloat(getAndroidVersion());
					if (androidVersion < 4.4) {
						$('body').addClass('android-lt-4-4'); // less than android 4.4
					}

					// variables
					numberInputType = 'text'; // don't use number input on android
					// appReviewUrl = 'https://play.google.com/store/apps/details?id=com.dannyconnell.android.fudget';
					appReviewUrl = 'market://details?id=com.dannyconnell.android.fudget';
					appReviewUrlFallback = 'https://play.google.com/store/apps/details?id=com.dannyconnell.android.fudget';
					moreAppsUrl = 'market://search?q=pub:Danny+Connell';
					moreAppsUrlFallback = 'https://play.google.com/store/apps/developer?id=Danny+Connell';
					feedbackUrl = 'mailto:fudget@dconnell.co.uk?subject=Fudget - Android - Feedback&body=' + getAndroidDeviceInfoForEmail();
					pageTransitionLength = 30;
					animateTutorialSlides = false;

					function getAndroidDeviceInfoForEmail() {

						if (window.cordova) {
							try {
								var deviceInfo = 	'\n\n';
									deviceInfo +=	'-- Please write message above this line --'
									deviceInfo +=	'\n\n';
									deviceInfo +=	'Platform: ' + device.platform + '\n';
									deviceInfo +=	'Platform version: ' + device.version + '\n';	
									deviceInfo +=	'Device model: ' + device.model + '\n';	
									deviceInfo +=	'Fudget version: ' + versionNumber;	

								var deviceInfoEncoded = encodeURI(deviceInfo);
							}
							catch(error) {
								var deviceInfoEncoded = '';
							}
						}
						else {
							deviceInfoEncoded = '';
						}
						
						return deviceInfoEncoded;
					}
				}

			}

		/*
			>> google analytics
		*/

			if(window.cordova && typeof window.analytics !== undefined) {
				window.analytics.startTrackerWithId("UA-71053803-3");
				// window.analytics.debugMode();
			} else {
				console.log("Google Analytics Unavailable");
			}

		/* 
			>> version number check 
		*/

			if (!localStorage['versionNumber']) {
				localStorage['versionNumber'] = versionNumber;
			}
			// if user has updated (new version)
			if (versionNumber != localStorage['versionNumber']) {
				// allow user to be prompted for feedback again
				localStorage.removeItem('stopAsking');
				localStorage.removeItem('tappedLater');
				// reset number of entries made
				localStorage.entriesMade = 0;
				localStorage['versionNumber'] = versionNumber;
				// allow user to be shown new version popup
				localStorage.removeItem('seenNewVersionPopup');
			}

			// add version number to settings page
			$('.version-number').text(versionNumber);

		/*
			>> setup pro version (if purchased) & check if pro trial expired
		*/

			// enableProVersion(); // DELETE

			if (localStorage['IAPfudgetProUpgrade']) {
				enableProVersion();
			}

			checkProTrialExpired();

		/* 
			>> fastclick - makes taps fire quicker on mobile
		*/

			FastClick.attach(document.body);
			
		/*
			>> initial login
		*/

			touchIdSupported = false;

			// check for touch id support
			if (isNative && platform == 'iOS') {
				touchid.checkSupport(
					function() {
						//touch id supported
						touchIdSupported = true;
					}
				);
			}

			// if there's a pin code in local storage
			if (isNative) {
				if (localStorage.pinCode !== undefined && localStorage.pinCode.length > 0) {
					doLogin();
				}
				// else, just hide the splash screen and get started
				else {
					setTimeout(function() {
				        navigator.splashscreen.hide();
				        setupAdmob();
				    }, 1000);
				}
			}


		/* 
			>> database setup
		*/
		
			balance.webdb.open();
			balance.webdb.createTables();
			// balance.webdb.restoreDatabase(balance.webdb.getAllBudgets);
			balance.webdb.getAllBudgets();
			balance.webdb.checkIfDropboxDataNewer();
			
	
		/*
			>> global variables
		*/
		
			currencySymbol = '$';
			if (localStorage.getItem('lastPage') === null) {
				localStorage.setItem('lastPage', 'budgets');
			}
			$info = $('#info');
			$proUpgradeBanner = $('.pro-upgrade-banner');
			$addEntry = $('#add-entry');
			$entriesTitleBarTitle = $('#entries-container .title-bar .title');
			$houseAdIframe = $('.house-ad-iframe');
			animating = false;
						
			
		/*
			>> instructions
		*/
		
			// if user has not swiped to settings yet, show 'settings this way' instruction
			if (localStorage.getItem('swipedToSettings') === null) {
				var $settingsThisWay = $('<p class="instruction">Settings this way <br> (swipe right)</p>');
				$settingsThisWay.prependTo('#budgets-container');				
			}
			
			// if user has not swiped to budgets yet, show 'budgets this way' instruction
			if (localStorage.getItem('swipedToMyFudgets') === null) {
				var $budgetsThisWay = $('<p class="instruction">Budgets this way <br> (swipe right)</p>');
				$budgetsThisWay.prependTo('#entries-container');				
			}


		/*
			>> settings - switches
		*/

			var labelSwitchEvent = (platform == 'Browser' ? 'click' : 'touchstart')

			$('.label-switch').on(labelSwitchEvent, function (e) {

				var $this = $(this);
				var $checkbox = $this.find('input[type=checkbox]');

				if (!$checkbox.is(':checked')) {
					$checkbox.attr('checked', 'checked');
				}
				else {
					$checkbox.removeAttr('checked');
				}

				setTimeout(function(){
					$checkbox.trigger('change');
				}, 100);

			});



			
		/*
			>> settings - set or change currency
		*/
		
			// if currency has already been set
			if (localStorage.getItem('currencySymbol') !== null) {
				currencySymbol = localStorage.getItem('currencySymbol');
			}
			// currency not yet set, set to dollar
			else {
				localStorage.setItem('currencySymbol', '$');
			}
			// select correct option in dropdown
			$('#currency-symbol option[value="' + currencySymbol + '"]').attr('selected', 'selected');
			currencySymbol = localStorage.getItem('currencySymbol');
			$('#currency-symbol-fake').text(currencySymbol);
		
			// user changes currency symbol drop down
			$('#currency-symbol').change(function (e) {
				e.preventDefault();
				var $this = $(this);
				var symbol = $this.val();
				defaultText = $('#currency-symbol-fake').text();
				// if user chose custom, show popup
				if (symbol === 'custom') {

					// prompt for currency symbol
					nativePrompt(
						'Enter your currency symbol',   // message
						customCurrencySymbolEntered,	// callback
						'Custom currency symbol',	    // title
						['Ok'],	                        // button labels (optional)
						defaultText		                // default text (optional)
					);

					// prompt for currency symbol callback
					function customCurrencySymbolEntered(result) {
						symbol = result.input1;
						$('#currency-symbol-fake').text(symbol);
						currencySymbol = symbol;
						localStorage.setItem('currencySymbol', currencySymbol);
						// set select box to blank option
						$('#currency-symbol').val('');
					}

				}
				// otherwise just set the symbol as selected
				else {
					$('#currency-symbol-fake').text(symbol);
					currencySymbol = symbol; 
					localStorage.setItem('currencySymbol', currencySymbol);	
				}


			});
			
			// disable ghost tap
			$('#currency-symbol').on('touchstart', function (e) {
				var $this = $(this);
				var $thisItem = $this.parents('.item');
				$thisItem.addClass('click-fixer');
				setTimeout(function(){
					$thisItem.removeClass('click-fixer');
				}, 1000);
			});

		/*
			>> settings - pin code lock
		*/

			// if there's a pinCode in localStorage, set PIN code setting to On
			if (localStorage.pinCode !== undefined && localStorage.pinCode.length > 0) {
				$('#pin-code-lock').attr('checked', 'checked');
			}

			// user changes pin code lock option
			$('#pin-code-lock').change(function (e) {
				// update option text
				if ($(this).is(':checked')) {
					promptForNewPinCode();
					function promptForNewPinCode() {
						window.plugins.pinDialog.prompt(
							"Please enter a passcode with at least 4 digits.",
							newPinEntered,
							"Enter passcode",
							["Cancel","OK"]
						);
					}
					function newPinEntered(results) {
						// OK clicked
					    if(results.buttonIndex === 2) {
					    	// if pin of 4 digits or more entered
					    	if (results.input1.length >= 4) {
					    		// enable touch id option
					    		$('#setting-touch-id-login').removeClass('disabled');
					    		// set pinCode localStorage value to entered pin
					    		localStorage.setItem('pinCode', results.input1);
					    		if (localStorage.emailAddress === undefined || localStorage.emailAddress === '') {
						    		$('#setting-email-address').trigger('click');
					    		}
					    	}
					    	// else prompt again
					    	else {
					    		promptForNewPinCode();
					    	}

					    }
					    // cancel clicked
					    if(results.buttonIndex === 1) {
					        // set select back to off
					        $('#pin-code-lock').removeAttr('checked');
					    }
					}
				}
				// if 'off', remove password from localStorage
				else {
					promptForPinCodeToDisable('Disable passcode lock');
					function promptForPinCodeToDisable(title) {
						window.plugins.pinDialog.prompt(
							"Please enter your passcode to disable passcode lock", 
							disablePinCodeLock, 
							title, 
							["Cancel","OK"]);
					}

					function disablePinCodeLock(results) {
						// if ok button clicked
						if(results.buttonIndex === 2) {
							// if entered pin matches stored pin
							if (localStorage.pinCode === results.input1) {
								localStorage.removeItem('pinCode');
								// disable touch id option
								$('#setting-touch-id-login').addClass('disabled');
								$('#touch-id-login').removeAttr('checked');
								localStorage.setItem('touchIdLogin', 'false');
							}
							// incorrect pin
							else {
								promptForPinCodeToDisable('Incorrect passcode');
							}
						}
						// cancel clicked - set option back to on
						else {
							$('#pin-code-lock').attr('checked', 'checked');
						}
					}
				}

			});


		/*
			>> settings - touch id login
		*/
		
			// if there's a touchIdLogin in localStorage, set touch id login setting to On
			if (localStorage.getItem('touchIdLogin') === 'true') {
				// $('#touch-id-login').val('On');
				$('#touch-id-login').attr('checked', 'checked');
				// $('#setting-touch-id-login .setting-option').text('On');
			}

			// show touch id login setting if supported
			setTimeout(function(){
				if (touchIdSupported) {		
					$('#setting-touch-id-login').removeClass('hidden');
				}
			}, 1000);

			// enable touch id login setting if there is a pin code set
			if (localStorage.pinCode !== undefined && localStorage.pinCode.length > 0) {
				$('#setting-touch-id-login').removeClass('disabled');
			}

			// user changes touch id login option
			$('#touch-id-login').change(function (e) {

				// if option disabled, warn user
				if ($(this).parent().parent().hasClass('disabled')) {
					e.preventDefault();
					nativeAlert(null, null, 'Please set a passcode first.');
					$(this).removeAttr('checked');
				}
				// else update
				else {			
					// if 'on', set localStorage value
					if ($(this).is(':checked')) {
						localStorage.setItem('touchIdLogin', 'true');
					}
					// else 'off', set localStorage value to false
					else {
						localStorage.setItem('touchIdLogin', 'false');
					}
				}

			});

		
		/*
			>> settings - email address
		*/

			// set on page load
			if (localStorage.emailAddress) {
				$('#setting-email-address .setting-option').text(localStorage.emailAddress);
			}

			// user clicks setting
			$('#setting-email-address').click(function (e) {

				// get users email address if already stored
				defaultText = localStorage.emailAddress || '';

				// prompt for email address
				nativePrompt(
					'Enter an email address in case you forget your passcode?',	// message
					emailPromptCallback,	                                    // callback
					'Enter email address',	                                    // title
					['Cancel', 'Ok'],	                                        // button labels (optional)
					defaultText,	                                            // default text (optional)
					null,                                                       // fallback function (optional)
					2                                                           // fallback ok button index (optional)
				);

				// prompt for email address callback
				function emailPromptCallback(result) {
					// ok
					if (result.buttonIndex === 2) {
						localStorage.emailAddress = result.input1
						$('#setting-email-address .setting-option').text(result.input1);
					}
				}

			});

			// if a passcode has been previously set, but user has never been prompted for an email address (cos they set passcode on previous version), prompt for email
			if (localStorage.pinCode !== undefined && localStorage.emailAddress === undefined) {
				$('#setting-email-address').trigger('click');
			}


		/*
			>> settings - theme submenu
		*/
		
			/* >>> show theme submenu */
			// this speeds up animations when the app first starts (adding budgets)
			$('.theme-submenu').transition({
				x: '100%'
			}, 200);
			$('.show-theme-submenu').click(function (e) {
				e.preventDefault();
				
				if (freeVersion) {
					e.preventDefault();
					// if nothing is animating
					if (animating === false) {
						animating = true;
						$('#no-ads-themes-popup').show().transition( {y: '0%'}, 300, function() {
							animating = false;
						});
					}
				}
				else {
					if (bodyDragging === false && animating === false) {
						animating = true;
						// scroll and offset
						scrollAndOffset('#settings-container');

						// move settings slightly to the left (ios7 parallax)
						$('#settings-container').transition({
							x: '-90px'
						}, 300);
						
						// show theme submenu
						$('.theme-submenu').show().transition({
							x: '0%'
						}, 300, function() {
							$('#settings-container').hide().css('top', '0px');
							$('.title-bar a.touch').removeClass('touch');
							animating = false;
						});
					}
				}
							
			});
			
			/* >>> back to settings from theme submenu */
			$('.theme-submenu').on('swiperight', function (e) {
				if (bodyDragging === false) {
					// scroll to top of the page, then offset budgets container so it doesn't suddenly 'jump'
					scrollAndOffset('.theme-submenu');
					// show settings
					$('#settings-container').show().transition({
						x: '0%'
					}, 300);
					// slide out budgets
					$('.theme-submenu').transition({
						x: '100%'
					}, 300, function() {
						$('.theme-submenu').hide().css('top', '0px');
						$('.title-bar a.touch').removeClass('touch');
					});
				}
			});	
			$('.theme-submenu .back-to-settings').click(function (e) {
				e.preventDefault();
				$('.theme-submenu').trigger('swiperight');
			});
			
			/* >>> change theme */
			$('.theme-submenu .item').click(function (e) {
				e.preventDefault();
				var $this = $(this);
				// get theme name/class
				var theme = $this.attr('data-theme');
				var themeTitle = $this.find('.theme-color').text();
				
				// change html class
				$('html').removeClass().addClass(theme);
				
				// set active item
				$('.theme-submenu .item.active').removeClass('active');
				$this.addClass('active');
				
				// update settings option
				$('.show-theme-submenu .setting-option').text(themeTitle);
				
				// add to localstorage
				localStorage.setItem('theme', theme);
			});
			
			/* >>> set theme on page load */
			if (localStorage.theme !== undefined) {
				// set active theme setting
				$('.theme-submenu .item.active').removeClass('active');
				$('.theme-submenu .item[data-theme=' + localStorage.theme + ']').addClass('active');
				// update settings option
				var themeTitle = $('.theme-submenu .item.active').find('.theme-color').text();
				$('.show-theme-submenu .setting-option').text(themeTitle);
			}
			

		/*
			>> settings - click 'tell a friend'
		*/

			$('.tell-a-friend').click(function (e) {

				// variables
				var message = 'Check out this app Fudget I\'ve been using to manage my money!';
				var subject = 'Fudget for iOS and Android. ';
				var imageUrl = 'http://fudget.dconnell.co.uk/assets/images/fudget-action-shot.png';

				gaTrackEvent('Settings', 'Settings > Tapped Tell a friend', null);

				// prompt for facebook, twitter, instagram or other
				nativeConfirm(
					'Share Fudget on...',	                            // message
					tellAFriendCallback,	                            // callback
					'Tell a friend about Fudget',	                	// title
					['Facebook', 'Twitter', 'Email, Message etc...'],	// button labels
					'Sharing only supported on native device.'	        // fallback message
				);

				// prompt callback
				function tellAFriendCallback(buttonIndex) {
			     	// facebook pressed
			     	if (buttonIndex == 1) {
			     		// share on facebook
			     		window.plugins.socialsharing.shareViaFacebook(
			     			'Fudget - fast, simple money management',
			     			null,
			     			appWebsiteUrl, // url
			     			function() {
			     				nativeAlert('Thanks for sharing!', null, 'Thanks!');
			     				gaTrackEvent('Settings', 'Settings > Tapped Tell a friend > Shared on Facebook', null);
			     			},
			     			function(errormsg){
			     			},
			     			null
			     		);						     		
			     	}
			     	// twitter pressed
			     	if (buttonIndex == 2) {
			     		// share on twitter
				     	window.plugins.socialsharing.shareViaTwitter(
				     		message + ' ' + appWebsiteUrl, 
				     		imageUrl /* img */, 
				     		null,
				     		function() {
				     			nativeAlert('Thanks for sharing!', null, 'Thanks!');
				     			gaTrackEvent('Settings', 'Settings > Tapped Tell a friend > Shared on Twitter', null);
				     		}, 
				     		function(errormsg){
				     		},
				     		null
				     	);
			     	}
			     	// other pressed
			     	if (buttonIndex == 3) {
			     		window.plugins.socialsharing.share(
			     			message + ' ' + appWebsiteUrl, 
			     			subject, 
			     			imageUrl,
			     			function() {
			     				nativeAlert('Thanks for sharing!', null, 'Thanks!');
			     				gaTrackEvent('Settings', 'Settings > Tapped Tell a friend > Shared on Email or Other', null);
			     			}, 
			     			function(errormsg){
			     			},
			     			null
			     		);
			     	}
				}



			});

		/*
			>> settings - click rate this app
		*/

			$('.rate-this-app').click(function(e) {
				e.preventDefault();
				// GA - track event
				gaTrackEvent('Settings', 'Settings > Tapped Write a review', null);
				// android
				if (platform === 'Android') {
					try {
					    window.open(appReviewUrl, '_system'); 
					}
					catch(err) {
					    window.open(appReviewUrlFallback, '_system'); 
					}
				}
				// ios
				else {
					window.open(appReviewUrl, '_system'); 
				}
			});

		/*
			>> settings - click email me
		*/

			$('.send-feedback').click(function(e) {
				e.preventDefault();
				// GA - track event
				gaTrackEvent('Settings', 'Settings > Tapped Email me', null);
				window.open(feedbackUrl, '_system'); 
			});

		/*
			>> settings - click more apps
		*/

			$('.more-apps').click(function(e) {
				e.preventDefault();
				// GA - track event
				gaTrackEvent('Settings', 'Settings > Tapped More apps', null);
				// android
				if (platform === 'Android') {
					try {
					    window.open(moreAppsUrl, '_system'); 
					}
					catch(err) {
					    window.open(moreAppsUrlFallback, '_system'); 
					}
				}
				// ios
				else {
					window.open(moreAppsUrl, '_system'); 
				}
			});

		/*
			>> settings - books
		*/

			$('.books').click(function(e) {
				e.preventDefault();
				// GA - track event
				gaTrackEvent('Settings', 'Settings > Tapped Books', null);
			    window.open(booksUrl, '_system'); 
			});

		/*
			>> settings - click clear data
		*/

			$('.clear-data').click(function(e) {
				e.preventDefault();

				// confirm before deleting localStorage and database
				nativeConfirm(
					'Are you sure you want to clear all data? This cannot be undone.',	// message
					confirmClearData,					                                // callback
					'Clear all data?',						                            // title
					['Clear data','Cancel'],							                // button labels
					null,									                            // fallback message 
					1,										                            // fallback - ok pressed
					2										                            // fallback - cancel pressed
				);

				function confirmClearData(buttonIndex) {
					// clear pressed
					if (buttonIndex === 1) {
						// clear localStorage
						localStorage.clear();
						// clear database
						balance.webdb.deleteDatabase();
					}
				}
			});


		/*
			>> touch feedback
		*/
		
			$('#settings .item, .settings .item, .button, .popup-close').on('touchstart', function (e) {
				var $this = $(this);
				$this.addClass('touch');
				if (platform === 'Android') {
					setTimeout(function(){
						$this.removeClass('touch');
					}, 1000);
				}
			});

			$('#settings .item, .settings .item, .button, .popup-close').on('touchend mouseleave', function (e) {
				var $this = $(this);
				$this.removeClass('touch');
			});


		/*
			>> initializations
		*/

			// hammer.js
			dragMinDistance = 1;

			// different hammer options for each platform
			if (platform === 'iOS') {
				hammerOptions = {
					drag_block_horizontal: true,
					drag_min_distance: dragMinDistance,
					drag_lock_to_axis: true,
					slow_touches: false
				}
			}
			else if (platform === 'Android') {
				hammerOptions = {
					drag_block_horizontal: true,
					drag_min_distance: dragMinDistance,
					// drag_lock_to_axis: true,
					slow_touches: false
				}
			}
			else {
				hammerOptions = {
					drag_block_horizontal: true,
					drag_min_distance: dragMinDistance,
					drag_lock_to_axis: true,
					slow_touches: false
				}
			}

			$("#container").hammer(hammerOptions);
			

			
			// dragging detection for iOS only
			bodyDragging = false; // for android & browser
			if (platform === 'iOS') {

				// check if body (viewport) is being dragged
				var bodyDragging = false;
				$('#entries-container, #budgets-container, #settings-container').live('dragup dragdown', function (ev) {
					bodyDragging = true;
				});
				$('#entries-container, #budgets-container, #settings-container').live('dragend', function (ev) {
					var t = setTimeout(function(){
						bodyDragging = false;
					}, 200);
				});

			}

					
		/*
			>> sortable
		*/

			sortable('#entries .entry');
			sortable('#budgets .budget');

			function sortable(element) {
				
				// tap and hold entry
				var originalOffset;
				var placeholderHeight;
				var $placeholder;

				var topCutOff = 0;
				var bottomCutOff = $(window).height() - $('#info').height() - 47 - 47;
				
				$(element + ':not(.editing)').live("hold", function(ev) {
					// only allow dragging if no options are showing
					if ($(element + '.showing-options').length < 1) {
						var $this = $(this);
						// create a placeholder
						placeholderHeight = $this.height();
						$placeholder = $('<div class="placeholder" style="height: ' + placeholderHeight + 'px"></div>');
						$this.after($placeholder);
						// get original y offset of entry
						originalOffset = $this.position().top;
						// add dragging class and pull out of the page
						$this.addClass('dragging').transit({scale: 1.05}).css('top', originalOffset + 'px');
						
					}
				});
				
				// drag entry
				var previousDraggedEntryOffsetY = 0;
				// cut off points for automatic scrolling (when user drags an entry near the top or bottom of the viewport)
				$(element).live("drag", function(ev) {
					$draggedEntry = $(this);
					draggedEntryHeight = $draggedEntry.height();
					// only if dragging class added by tap hold
					if ($draggedEntry.hasClass('dragging')) {
						ev.gesture.preventDefault();
						draggedEntryOffsetY = originalOffset + ev.gesture.deltaY;
						// determine drag direction
						if (draggedEntryOffsetY <= previousDraggedEntryOffsetY) {
							direction = 'up';
						}
						else {
							direction = 'down';
						}
						previousDraggedEntryOffsetY = draggedEntryOffsetY;
						// move entry to cursor / touch position
						$draggedEntry.css({
							top: draggedEntryOffsetY
						});
						// compare position with the position of every entry
						$(element).not($draggedEntry).each(function(index) {
							$compareEntry = $(this);
							compareEntryOffset = $compareEntry.position().top;
							// if dragged entry is 'near' another entry, swap their places
							if (draggedEntryOffsetY > (compareEntryOffset - ((draggedEntryHeight / 2) - 10)) && draggedEntryOffsetY < (compareEntryOffset + ((draggedEntryHeight / 2) - 10)) ) {
								if (direction === 'up') {
									$placeholder.insertBefore( $compareEntry );
								}
								else {
									$placeholder.insertAfter( $compareEntry );
								}
							}
						});
						
						// scroll viewport up or down if user near the top or bottom
						var currentScrollTop = $(window).scrollTop(); // how far down the page the viewport is from the top
						var currentDragPosition = draggedEntryOffsetY - currentScrollTop; // y position of the dragged entry in relation to viewport
						if (currentDragPosition < topCutOff) {
							$("html,body").scrollTop(currentScrollTop - 5);
						}
						if (currentDragPosition > bottomCutOff) {
							$("html,body").scrollTop(currentScrollTop + 5);
						}
						
					}
				// user lets go of dragged entry
				}).live("release", function(ev) {
					var $droppedEntry = $(this);
					if ($droppedEntry.hasClass('dragging')) {
						// get position of placeholder
						placeholderY = $placeholder.position().top;
						// animate dropped entry to placeholder position
						$droppedEntry.animate({
							top: placeholderY
						}, 100, function() {
							// scale back down
							$droppedEntry.transit({
								scale: 1
							}, 100, function() {
								$droppedEntry.removeClass('dragging');
								$droppedEntry.insertAfter($placeholder);
								$droppedEntry.removeAttr('style');
								$placeholder.remove();

								// update entry order numbers (if an entry was released)								
								if (element === '#entries .entry') {
									// update order numbers for this particular budget - create arrays of budgetIds, entryIds and orderNumbers
									var entryIds = [],
										orderNumbers = [],
										budgetIds = [];
									$(element).each(function(index) {
										entryId = $(this).attr('data-id');
										entryIds.push(entryId);
										orderNumber = index+1;
										orderNumbers.push(orderNumber);
										budgetId = $('.budget.active').attr('data-id');
										budgetIds.push(budgetId);
										
										balance.webdb.updateEntryOrderNumbers(entryIds, orderNumbers, budgetIds);
									});
								}
								
								// update budget order numbers (if a budget was released)								
								if (element === '#budgets .budget') {
									// update budget order numbers - create arrays of budgetIds and orderNumbers
									var budgetIds = [],
										orderNumbers = []
									$(element).each(function(index) {
										budgetId = parseInt($(this).attr('data-id'));
										budgetIds.push(budgetId);
										orderNumber = index+1;
										orderNumbers.push(orderNumber);
									});
									balance.webdb.updateBudgetOrderNumbers(budgetIds, orderNumbers);
								}
								
								
							});
						});
					}
					
				});
				
			}


		/*
			>> show budget list on swipe right
		*/
		
			$('#entries-container').on('swiperight', function (e) {
				showBudgetsList();
			});
			
			// click back button
			$('.back-to-budgets').on('click', function (e) {
				e.preventDefault();
				showBudgetsList();
			});

			function showBudgetsList() {
				hasHistory = true;
				var $this = $('#entries-container');
				
				if (!$('#entries .entry.showing-options').length > 0 && !$('#entries .entry.editing').length > 0 && bodyDragging === false) {

					// GA - track page
					gaTrackView('Budgets');

					$info.addClass('off-bottom');

					// scroll to top of the page, then offset entries container so it doesn't suddenly 'jump'
					scrollAndOffset('#entries-container');
					
					// show budgets container
					$('#budgets-container').show().transition({
						x: '0%'
					}, pageTransitionLength);
					
					// slide out the entries container
					$('#entries-container').transition({
						x: '100%'
					}, pageTransitionLength, function() {
						$('#entries-container').hide().css('top', '0px');
						$('#entries').html('');
						$('.title-bar a.touch').removeClass('touch');
						localStorage.setItem('lastPage', 'budgets');
						// remove styles from #budgets-container
						$('#budgets-container').attr('style', 'display: block;');
						// show pro upgrade banner
						showProUpgradeBanner();
						// hide 'my fudgets this way' instruction forever
						if (localStorage.getItem('swipedToMyFudgets') === null) {
							$budgetsThisWay.remove();
							localStorage.swipedToMyFudgets = true;
						}
						// hide feedback module if visible
						if ($('body').hasClass('show-feedback-module')) {
							$('body').removeClass('show-feedback-module');
						}
					});
				}				
			}
		
			
		/*
			>> show settings on swipe right
		*/
			
			// this speeds up animations when the app first starts (adding budgets)
			$('#budgets-container').transition({
				x: '0%'
			}, 200);
			
			$('#budgets-container').on('swiperight', function (e) {
				
				if (!$('#budgets .budget.showing-options').length > 0 && !$('#budgets .budget.editing').length > 0 && bodyDragging === false) {
					hasHistory = true;
					// GA - track page
					gaTrackView('Settings');
					// hide pro upgrade banner
					hideProUpgradeBanner();
					// scroll to top of the page, then offset budgets container so it doesn't suddenly 'jump'
					scrollAndOffset('#budgets-container');
					// show settings
					$('#settings-container').show().transition({
						x: '0%'
					}, pageTransitionLength);
					// slide out budgets
					$('#budgets-container').transition({
						x: '100%'
					}, pageTransitionLength, function() {
						$('#budgets-container').hide().css('top', '0px');
						localStorage.setItem('lastPage', 'settings');
						$('.title-bar a.touch').removeClass('touch');
						// remove styles from settings container to prevent loss
						// of sticky title bar on android
						$('#settings-container').removeAttr('style');
						// hide 'settings this way' instruction forever
						if (localStorage.getItem('swipedToSettings') === null) {
							$settingsThisWay.remove();
							localStorage.swipedToSettings = true;
						}
					});
				}
						
			});	
			
			$('#budgets-container .back-to-settings').on('click', function (e) {
				e.preventDefault();
				$('#budgets-container').trigger('swiperight');
			});
			
		/*
			>> show budgets on swipe left
		*/
		
			$('#settings-container').on('swipeleft', function (e) {
				
				if (bodyDragging === false && animating === false) {
					animating = true;

					hasHistory = true;

					// GA - track page
					gaTrackView('Budgets');

					// scroll to top of the page, then offset budgets container so it doesn't suddenly 'jump'
					scrollAndOffset('#settings-container');

					// move settings slightly to the left (ios7 parallax)
					$(this).transition({
						x: '-90px'
					}, pageTransitionLength);
					
					// show budgets container
					$('#budgets-container').show().transition({
						x: '0%'
					}, pageTransitionLength, function() {
						$('#settings-container').hide().css('top', '0px');
						localStorage.setItem('lastPage', 'budgets');
						$('.title-bar a.touch').removeClass('touch');
						// show pro upgrade banner
						showProUpgradeBanner();
						// remove styles from #budgets-container
						$('#budgets-container').attr('style', 'display: block;');
						animating = false;
					});
				}
						
			});
			
			$('#show-budgets-container, .forward-to-budgets').on('click', function (e) {
				e.preventDefault();
				var $this = $(this);
				$('#settings-container').trigger('swipeleft');
			});
			

			
		/*
			>> tap budget - show entry list
		*/
		
			$('#budgets .budget:not(.editing,.showing-options) .budget-details').live('click', function (e, isTriggered) {
				e.preventDefault();
				hasHistory = true;
				// GA - track page
				gaTrackView('Entries');
				// if no budget options are showing, and nothing is animating, and item not being dragged, show entry list
				if ($('#budgets .budget.showing-options').length < 1 && animating === false && !$(this).parent().hasClass('dragging')) {
					animating = true;
					var $this = $(this);
					var $thisBudget = $this.parent();
					// touch feedback
					$thisBudget.addClass('touch');
					$('#budgets .budget.active').removeClass('active');
					$this.parent().addClass('active');
					var budgetId = $this.parent().attr('data-id');
					var budgetTitle = $this.find('textarea').val();
					var $entriesContainer = $('#entries-container');
					var $budgetsContainer = $('#budgets-container');
					
					// hide pro upgrade banner
					hideProUpgradeBanner();
					
					// scroll to top of the page, then offset budgets container so it doesn't suddenly 'jump'
					/// scrollAndOffset('#budgets-container');
					
					finishedLoadingEntries = false;

					balance.webdb.getAllEntries(budgetId, loadAllEntries);

					// wait for entries to load into markup
					var waitForEntries = setInterval(function(){
						if (finishedLoadingEntries) {
							displayEntries();
							clearInterval(waitForEntries);
						}
					}, 100);

					function displayEntries() {
						
						// setup textareas
						$('#entries .entry textarea').each(function(index) {
							initializeAutogrowTextarea($(this));
						});
						$entriesTitleBarTitle.text(budgetTitle);
						// if event was triggered by javascript (not a human) fade entries in instead of slide in
						if (isTriggered) {
						    $entriesContainer.hide().transition({x: '0%'});
						    $entriesContainer.fadeIn();
						    transitionTime = 0;
						}
						else {
							$entriesContainer.show();
							transitionTime = 300;
						}

						// prompt for feedback if created over x entries
						if (localStorage.entriesMade >= showRateThisAppPopupAfterEntries && !localStorage.stopAsking) {
							// if user tapped Later, just show the review icon
							if (localStorage.tappedLater) {
								showReviewIcon();
							}
							// else prompt for feedback
							else {
								setTimeout(function(){
									promptForFeedback();
								}, 1000);
							}
						}

						// hide #budgets
						$('#budgets, #budgets-container .buttons, #budgets-container .title-bar').addClass('hidden');

						setTimeout(function(){
							
							// scroll to bottom
							scrollToBottom();

							// ios - fancy transition
							if (platform === 'iOS') {

								// move budgets container just a little (ios 7 style)
								$budgetsContainer.transition({
									x: '-90px'
								}, 0);

								// slide in entries container
								$entriesContainer.transition({
									x: '0%'
								}, transitionTime, function() {
									finishBudgetsToEntriesTransition();
								});

							}
							// android - simple transition
							else {
								finishBudgetsToEntriesTransition();
							}

						}, 200);


						function finishBudgetsToEntriesTransition() {
							// show & reset #budgets
							$('#budgets, #budgets-container .buttons, #budgets-container .title-bar').removeClass('hidden');
							$budgetsContainer.hide().css('top', '0px');
							$thisBudget.removeClass('touch');

							// show #info
							$info.removeClass('off-bottom');

							// remove styles from #entries-container
							$('#entries-container').attr('style', 'display: block;');

							animating = false;
							localStorage.setItem('lastPage', 'entries');
							localStorage.setItem('lastBudgetId', budgetId);
						}

					}
					
				}
				
			});


	    /* 
	    	>> autogrow textareas
	    */
	    
			$('#entries textarea.entry-title').each(function(index) {
				textarea = $(this);
				initializeAutogrowTextarea(textarea);
			});				
				

		/*
			>> pro upgrade banner
		*/
		
			// setup pro upgrade banner
			$('.pro-upgrade-banner-slides').slick({
				autoplay: true,
				autoplaySpeed: 4500,
				dots: true,
				arrows: false
			});
		
			// close pro upgrade banner
			$('.pro-upgrade-banner .icon-delete').click(function (e) {
				e.preventDefault();
				e.stopPropagation();
				hideProUpgradeBanner();
				setTimeout(function(){
					$proUpgradeBanner.addClass('hidden');
				}, 300);
			});
	    /* 
	    	>> tap or blur entry title
	    */
	    
			$('#entries p.entry-title').live('click', function(e) { 
				var $this = $(this);
				// don't allow if options are showing
				if ($('#entries .entry.showing-options').length < 1) {
					
					// add focus class to entry
					$this.parent().parent().addClass('focus-left');
					
			    	$this.addClass('invisible');
			    	$this.parent().find('textarea.entry-title').removeClass('off-page').select();
			    }
			});
		    $('#entries textarea.entry-title').live('blur', function (e) {
		    	var $this = $(this);
		    	var $entry = $this.parent().parent();
		    	
		    	// show info box and title bar
		    	setTimeout(function(){
		    		if ($('#entries-container .focus-left, #entries-container .focus-right, #entries-container .focus-all').length === 0) {
				    	$info.removeClass('off-bottom');
				    	// change position of title bar back to fixed
				    	$('#entries-container .title-bar').removeClass('pos-absolute');
				    	$('#entries').removeClass('padding-top-50');
				    }
		    	}, 200);
		    	
		    	// remove focus class from entry
		    	$this.parent().parent().removeClass('focus-left');
		    	
				// $addEntry.removeClass('disabled');
		    	$this.addClass('off-page');
		    	$this.parent().find('p.entry-title').removeClass('invisible');
		    	$entry.removeClass('editing');

		    	$houseAdIframe.removeClass('hidden');
		    });
		    $('#entries textarea.entry-title').live('focus', function (e) {
				var $this = $(this);
				
				// select all text
				/*this.selectionStart = 0;
				this.selectionEnd = 999;*/
				
				// add focus class to entry
				$this.parent().parent().addClass('focus-left');
				
				// hide info box
				$info.addClass('off-bottom');
				
				// change title bar position to absolute 
				$('#entries-container .title-bar').addClass('pos-absolute');
				$('#entries').addClass('padding-top-50');
				
		    	$this.removeClass('off-page');
		    	$this.parent().find('p.entry-title').addClass('invisible');		    	
		    	$this.parent().parent().addClass('editing');

		    	$houseAdIframe.addClass('hidden');
		    });
		    $('#entries textarea.entry-title').live('keyup', function (e) {
		    	var $this = $(this);
		    	var titleText = $this.val();
		    	$this.parent().find('p.entry-title').text(titleText);
		    });
		    
	 
	    /* 
	    	>> tap or blur entry amount
	    */
	    
		    $('#entries p.amount').live('click', function (e) {
		    	var $this = $(this);
		    	var $entry = $this.parent().parent();
		    	var $entryDetails = $entry.find('.entry-details');
				// don't allow if options are showing
				if ($('#entries .entry.showing-options').length < 1 && $entry.css('translate') !== null && !$entry.hasClass('dragging-x')) {
					
					// add focus class to entry
					$this.parent().parent().addClass('focus-right');
			    	
			    	$this.addClass('hidden');
			    	var $inputAmount = $this.parent().find('input.amount');
			    	// if the value is zero, empty input, rather than having '0' which the user would then have to delete
			    	if ($inputAmount.val() === 0) {
			    		$inputAmount.val('');
			    	}
			    	$inputAmount.removeClass('invisible').select();
			    }
		    });    	
			$('#entries input.amount').live('blur', function (e) {
		    	$this = $(this);
		    	// remove focus class from entry
		    	$this.parent().parent().removeClass('focus-right');
		    	
		    	// show info box and title bar
		    	setTimeout(function(){
		    		if ($('#entries-container .focus-left, #entries-container .focus-right, #entries-container .focus-all').length === 0) {
				    	$info.removeClass('off-bottom');
				    	// change position of title bar back to fixed
				    	$('#entries-container .title-bar').removeClass('pos-absolute');
				    	$('#entries').removeClass('padding-top-50');
				    }
		    	}, 200);
		    	
		    	$this.addClass('invisible');
		    	
		    	// update 'fake' input
		    	fakeInput = $this.parent().find('p.amount');
		    	fakeInput.removeClass('hidden');
		    	if (isNaN($this.val()) || $this.val() === '') {
		    		$this.val(0);
		    	}
		    	fakeInputText = $this.val();
		    	fakeInputText = currencify(fakeInputText);
		    	
		    	// add plus or minus
		    	var modifier;
		    	if ($this.parent().parent().hasClass('income')) {
		    		modifier = '+';
		    	}
		    	else {
		    		modifier = '-';	
		    	}
		    	fakeInputText = modifier +  ' ' + fakeInputText;
		    	fakeInput.find('span').html(fakeInputText);
		    	$this.parent().parent().removeClass('editing');

		    	$houseAdIframe.removeClass('hidden');
		    });
			$('#entries input.amount').live('focus', function (e) {
		    	var $this = $(this);
		    	
		    	// add focus class to entry
		    	$this.parent().parent().addClass('focus-right');
		    	
				// hide info box
				$info.addClass('off-bottom');
				
				// change title bar position to absolute 
				$('#entries-container .title-bar').addClass('pos-absolute');
				$('#entries').addClass('padding-top-50');
				
		    	$this.removeClass('invisible');
		    	$this.parent().find('p.amount').addClass('hidden');
		    	$this.parent().parent().addClass('editing');

		    	$houseAdIframe.addClass('hidden');
		    });
		    
		    
	    /* 
	    	>> blur entry (update)
	    */
	    
	    	// get original value on focus
	    	var originalValue = '';
	    	$('#entries textarea.entry-title, #entries input.amount').live('focus', function (e) {
	    		originalValue = $(this).val();
	    	});

	    	$('#entries textarea.entry-title, #entries input.amount').live('blur', function (e) {
		    	// update entry in database
		    	var $this = $(this);
		    	var $entry = $this.parent().parent();
		    	var entryIndex = $entry.index() + 1;
		    	var title = $entry.find('textarea.entry-title').val();
		    	var amount = $entry.find('input.amount').val();
		    	var isexpense = 0;
		    	var blurredValue = $(this).val();
		    	var pinned;
		    	var checkForEntryId;
		    	var entryId;

				// if data-id attribute is empty (because we're still waiting for entry to be added to db - and attribute to be updated) then wait for a data-id attribute
				if ($entry.attr('data-id') === '') {
					var checkForEntryIdCount = 0;
					checkForEntryId = setInterval(function(){
						checkForEntryIdCount++;
						if (checkForEntryIdCount === 50) {
							clearInterval(checkForEntryId);
						}
						else if ($entry.attr('data-id') !== '') {
							clearInterval(checkForEntryId);
							entryId = $entry.attr('data-id');
							finishUpdatingEntry(entryId, title);
						}
					}, 100);
				}
				else {
					entryId = $entry.attr('data-id');
					finishUpdatingEntry(entryId, title);
				}

				function finishUpdatingEntry() {

			    	// if amount is null or NaN, set it to 0
			    	if (isNaN(amount) || amount === '') {
			    		amount = 0;
			    	}
			    	var currentlyViewedBudgetId = $('#budgets .budget.active').attr('data-id');
			    	// if its an expense, make positive number negative and set isexpense value
			    	if ($entry.hasClass('expense')) {
			    		amount = amount * -1; // make positive number negative
			    		isexpense = 1;
			    	}	
			    	// entry NOT pinned, just update it
			    	if (!$entry.hasClass('pinned')) {
			    		pinned = 0;
			    		// update entry
				    	balance.webdb.editEntry(entryId, title, amount, pinned, isexpense);
					}
			    	// entry IS pinned: unpin first, then update that
			    	else if (blurredValue !== originalValue) { // only if value has changed
			    		pinned = 1;
			    		// show loading screen
			    		showLoadingScreen();
			    		// unpin entry
			    		balance.webdb.unpinEntry(entryId, currentlyViewedBudgetId);
			    		// update entry
			    		var t = setTimeout(function(){
			    			$entry = $('#entries .entry:nth-child(' + entryIndex + ')');
			    			// add pinned class
			    			$entry.addClass('pinned');
			    			$entry.find('.pin').addClass('unpin').removeClass('pin');
			    			// update entry
					    	var entryId = $entry.attr('data-id');
					    	// update title elements
					    	$entry.find('.entry-title').text(title);
					    	// update amount elements
					    	$inputAmount = $entry.find('input.amount');
					    	// make sure value that gets added to input value attribute is positive
					    	if (amount < 0) {
					    		inputValueAmount = amount * -1;	
					    	}
					    	else {
					    		inputValueAmount = amount;
					    	}
					    	$inputAmount.val(inputValueAmount);
					    	fakeInput = $inputAmount.parent().find('p.amount');
					    	fakeInputText = $inputAmount.val();
					    	fakeInputText = Math.abs(fakeInputText);
					    	fakeInputText = currencify(fakeInputText);
					    	
					    	// add plus or minus
					    	var modifier;
					    	if ($inputAmount.parent().parent().hasClass('income')) {
					    		modifier = '+';
					    	}
					    	else {
					    		modifier = '-';
					    	}
					    	fakeInputText = modifier +  ' ' + fakeInputText;
					    	fakeInput.html(fakeInputText);
					    	
					    	// update entry in database
					    	balance.webdb.editEntry(entryId, title, amount, pinned, isexpense);
					    	
					    	// update income and balance
					    	calculateIncomeAndBalance();
					    	
					    	// hide loading screen
					    	hideLoadingScreen();

			    	    	// android - scroll to bottom
			    	    	// if (platform === 'Android') {
			    	    	// 	setTimeout(function(){
			    	    	// 		scrollToBottom();
			    	    	// 	}, 400);
			    	    	// }
			    		// }, 250);
			    		}, 500);
			    	}

			    	// if android, refresh banner
			    	if (platform === 'Android') {
			    		refreshBanner();
			    	}

			    }

			});
			

		/*
			>> change entry date
		*/

			$('.entry-date').live('blur', function() {

				var $this = $(this),
					$entry = $this.parent().parent(),
					budgetId = parseInt($('#budgets .budget.active').attr('data-id')),
					entryId = parseInt($entry.attr('data-id')),

					// get unix date from date input value
					date = $this.val(),
					dateUnix = dateToUnix(date);

				// add date to database
				balance.webdb.editDate(budgetId, entryId, dateUnix);

				// change position of title bar back to fixed
				$('#entries-container .title-bar').removeClass('pos-absolute');
				$('#entries').removeClass('padding-top-50');

			})

			$('.entry-date').live('focus', function() {

				var $this = $(this);

				// remove placeholder
				$this.removeAttr('placeholder');

				// change title bar position to absolute 
				$('#entries-container .title-bar').addClass('pos-absolute');
				$('#entries').addClass('padding-top-50');
			})

			$('.entry-date').live('change', function() {

				var $this = $(this);
				$this.trigger('blur');

			})


		
		/*
			>> calculator
		*/
		
			// show calculator
			$('.entry-options .calculator').live('click', function (e) {
				e.preventDefault();
				var $this = $(this);

				// if locked, show calculator upgrade popup
				if (freeVersion) {
					$('#calculator-popup').show().transition( {y: '0%'}, 300, function() {
		 				animating = false;
		 			});
				}
				// else show calculator
				else {
					var $entry = $this.parents('.entry');
					var $amount = $entry.find('input.amount');
					var startingValue = parseFloat($amount.val());
					$amount.addClass('using-calc');
					
					initCalculator(startingValue);
					$('.calculator-container').removeClass('hidden');
					setTimeout(function(){
						$('.calculator-container').addClass('show');
					}, 100);
				}
			});
			
			// close calculator
			$('.close-calculator').live('click', function (e) {
				e.preventDefault();
				$('.calculator-container').removeClass('show');
				// destroy calculator
				setTimeout(function(){
					$('.calculator-container .calculator').calculator('destroy');
					$('.using-calc').removeClass('using-calc');
					$('.calculator-container').addClass('hidden');
				}, 350);
			});
			
			// touch feedback
			$('button').live('click', function (e) {
				var $this = $(this);
				$this.addClass('touch');
				setTimeout(function(){
					$this.removeClass('touch');
				}, 500);
			});
		
			function initCalculator(value) {
			
				$('.calculator-container .calculator').calculator({
					onClose: useValue,
					layout: ['_7_8_9_/',
							'_4_5_6_*',
							'_1_2_3_-',
							'_0_._+',
							'@XCABS@U_='],
					clearText: '<i class="icon-bin"></i>',
					backspaceText: '<i class="icon-backspace"></i>',
					useText: '<i class="icon-tick"></i>',
					value: value,
					precision: 5,
					onButton: function(label, value, inst) { 
				        switch (label) {
				        	case '/':
				        		activateCalcButton('.calculator-divide');
				        		break;
				        	case '*':
				        		activateCalcButton('.calculator-multiply');
				        		break;
				        	case '-':
				        		activateCalcButton('.calculator-subtract');
				        		break;
				        	case '+':
				        		activateCalcButton('.calculator-add');
				        		break;
				        	default:
				        		break;
				        }
				        function activateCalcButton(selector) {
	                		setTimeout(function(){
	        	        		$(selector).addClass('active');
	                		}, 25);
	                	}
				    }
				});
				function useValue(value, inst) { 
					// update value of entry amount and trigger blur so everything updates
				    var valueFixed = Math.abs(value.toFixed(2));
				    var $amount = $('.using-calc');
				    var $entry = $amount.parents('.entry');
				    $amount.val(valueFixed).trigger('blur');
				    // close the calculator
				    $('.close-calculator').trigger('click');
				    // hide the options
				    setTimeout(function(){
					    $entry.find('.entry-details').transition({
					    	x: '0'
					    }, 100, function() {
					    	$entry.removeClass('showing-options');
					    });
				    }, 300);
				}
			}	
			
		
		/* 
			>> calculate total income 
		*/
		
			calculateIncomeAndBalance();
			$('#entries input.amount').live('blur change', function (e) {
				setTimeout(function(){
					calculateIncomeAndBalance();
				}, 200);
			});
			
			
		/*
			>> tap add entry
		*/
		
			$('#add-income, #add-expense').on('click', function (e) {
				e.preventDefault();

				// figure out type of entry to add
				var $this = $(this);
				var entryType;
				if ($this.is('#add-income')) {
					entryType = 'income';
				}
				else {
					entryType = 'expense';	
				}
				
				// hide any showing entry options
				if ($('#entries .item.showing-options').length > 0) {
					$('#entries .item.showing-options .entry-details').transition({
						x: '0'
					}, 100, function() {
						$('#entries .item.showing-options').removeClass('showing-options');
						addNewEntryMarkup();
					});
				}
				else {
					addNewEntryMarkup();	
				}
				
				function addNewEntryMarkup() {
				
					// add markup
					var operator;
					var isExpense;
					if (entryType === 'expense') {
						operator = '-';
						isExpense = 1;
					}
					else {
						operator = '+';	
						isExpense = 0;
					}
					var placeholderText;
					if (entryType === 'expense') {
						placeholderText = 'E.g. Rent';
					}
					else {
						placeholderText = 'E.g. Salary';	
					}
					
					var entryMarkup = 	'<div class="entry ' + entryType + ' item clearfix clear" data-id="">';
					entryMarkup += 			'<div class="entry-details item-details clearfix theme-color theme-border">';
					entryMarkup += 				'<textarea class="entry-title item-title theme-color" rows="1" cols="20" autocapitalize="on" autocorrect="off" autocomplete="off" placeholder="' + placeholderText + '"></textarea>';
					entryMarkup += 				'<p class="entry-title item-title fake theme-color"></p>';
					entryMarkup += 				'<input class="amount item-title invisible theme-color" type="' + numberInputType + '" value="" autocapitalize="off" autocorrect="off" autocomplete="off">';
					entryMarkup += 				'<p class="amount item-title fake theme-color"><span>' + operator + ' ' + currencySymbol + ' 0.00</span></p>';
					entryMarkup +=				'<input type="date" class="entry-date theme-color" placeholder="Date">';
					entryMarkup += 			'</div>';
					entryMarkup += 			'<div class="entry-options item-options theme-bg theme-border">';
					entryMarkup += 				'<div class="delete option">';
					entryMarkup += 					'<div class="icon">';
					entryMarkup +=						'<i class="icon-bin"></i>';
					entryMarkup +=						'<span>Delete</span>';
					entryMarkup +=					'</div>';
					entryMarkup +=				'</div>';
					entryMarkup += 				'<div class="pin option">';
					entryMarkup += 					'<div class="icon">';
					entryMarkup +=						'<i class="icon-star"></i>';
					entryMarkup +=						'<span>Star</span>';
					entryMarkup +=					'</div>';
					entryMarkup +=				'</div>';
					entryMarkup += 				'<div class="unpaid option">';
					entryMarkup += 					'<div class="icon"><i class="icon-unpaid"></i>';
					entryMarkup += 					'<span>Paid</span></div>';
					entryMarkup += 				'</div>';
					entryMarkup += 				'<div class="calculator option disabled-on-free">';
					entryMarkup += 					'<div class="icon">';
					entryMarkup +=						'<i class="icon-calculator"></i>';
					entryMarkup +=						'<span>Calc.</span>';
					entryMarkup +=					'</div>';
					entryMarkup +=				'</div>';				
					entryMarkup += 			'</div>';
					entryMarkup += 		'</div>';
					$entryMarkup = $(entryMarkup);
					$entryMarkup.appendTo('#entries');
				
					// select and setup textarea
					var textarea = $entryMarkup.find('textarea.entry-title');
					textarea.select();
					initializeAutogrowTextarea(textarea);
					updateEntriesMade(); // update count of entries made, so we can ask user to rate app after 50		
					
					// $info.hide();
					$info.addClass('off-bottom');

					// setup date fields
					// datePicker = $entryMarkup.find('.entry-date').pickadate(pickadateOptions);
				
					// add entry to database
					var budgetId = $('#budgets .budget.active').attr('data-id');
					var orderNumber = parseInt($('#entries .entry').length, 10) + 1;
					balance.webdb.addEntry(budgetId, orderNumber, isExpense, $entryMarkup);

					// android - scroll down to bottom
					if (platform === 'Android') {
						setTimeout(function(){
							scrollToBottom();
						}, 400);
					}

				}
				
			});	
						

		/*
			>> swipe left - show options
		*/
			
			var draggedEntryOffsetX;

			// figure out at which point on the left drag we should allow swipe-to-delete
			var swipeToDeleteOffset = -220; // iPhone 4 + 5
			if ($(window).width() >= 375) {
				swipeToDeleteOffset = -275; // iPhone 6
			}
			if ($(window).width() >= 414) {
				swipeToDeleteOffset = -314; // iPhone 6 Plus
			}
			if ($(window).width() >= 768) {
				swipeToDeleteOffset = -384; // iPad portrait
			}

			// Android - item swipe events (drag events didn't work too well)
			/*if (platform === 'Android') {

				// swipe entry left
				$('#entries .entry:not(.editing), #budgets .budget:not(.editing)').live("swipeleft", function(ev) {
					var $this = $(this);
					var $entryDetails = $this.find('.entry-details, .budget-details');
					if ($('#entries .showing-options, #budgets .showing-options').length < 1 && $('#entries .editing, #budgets .editing').length < 1 && animating === false) {
						$entryDetails.transition({
							x: '-165'
						}, 100, function() {
							$this.addClass('showing-options');
						});
					}
				});

				// swipe entry right
				$('#entries .entry.showing-options:not(.editing), #budgets .budget.showing-options:not(.editing)').live("swiperight", function(ev) {
					var $this = $(this);
					var $entryDetails = $this.find('.entry-details, .budget-details');
					$entryDetails.css({x: 0});
					$this.removeClass('showing-options');
				});

			}*/

			// iOS - item drag events (more iOS style item drags)
			// if (platform === 'iOS') {

				// drag entry left
				$('#entries .entry:not(.editing), #budgets .budget:not(.editing)').live("dragleft", function(ev) {
					var $this = $(this);
					// log(ev.gesture.angle);
					// console.log(ev.gesture.angle);
					// only fire if user isn't swiping vertically quite a bit
					// if (ev.gesture.deltaY <= 30 && ev.gesture.deltaY >= -30) {
					// if (bodyDragging === false) {

						// if no entry or budget options are showing & nothing is being edited, drag the entry details along and uncover options
						if ($('#entries .showing-options, #budgets .showing-options').length < 1 && $('#entries .editing, #budgets .editing').length < 1 && bodyDragging === false && !$this.hasClass('dragging') && animating === false) {
							$this.addClass('dragging-x');
							var $entryDetails = $this.find('.entry-details, .budget-details');
							draggedEntryOffsetX = ev.gesture.deltaX;
							if (draggedEntryOffsetX > (swipeToDeleteOffset - 20)) {
								$entryDetails.css({x: draggedEntryOffsetX});
							}
							// swipe to delete
							if (draggedEntryOffsetX < swipeToDeleteOffset) {
								$this.addClass('swipe-to-delete');	
							}
							else {
								$this.removeClass('swipe-to-delete');	
							}
						}
					// }
				});
				
				// drag entry right
				$('#entries .entry.showing-options:not(.editing), #budgets .budget.showing-options:not(.editing)').live("dragright", function(ev) {
					var $this = $(this);
					// if nothing is being edited
					if ($('#entries .editing, #budgets .editing').length < 1 && bodyDragging === false && !$this.hasClass('dragging') && animating === false) {
						$this.addClass('dragging-x');
						var $entryDetails = $this.find('.entry-details, .budget-details');
						originalOffset = -181;
						draggedEntryOffsetX = originalOffset + ev.gesture.deltaX;
						if (draggedEntryOffsetX > -211 && draggedEntryOffsetX < 0) {
							$entryDetails.css({x: draggedEntryOffsetX});
						}
					}
				});
				
				// drag end
				$('#entries .entry.dragging-x:not(.editing), #budgets .budget.dragging-x:not(.editing)').live("dragend", function(ev) {
					var $this = $(this);
					var $entryDetails = $(this).find('.entry-details, .budget-details');
					// if no options are showing 
					if ($('#entries .showing-options, #budgets .showing-options').not(this).length < 1 && !$this.hasClass('dragging') && animating === false) {
						// if dragged enough to show delete button alone, delete entry
						if (draggedEntryOffsetX < swipeToDeleteOffset) {
							$this.addClass('swipe-to-delete');	
							$this.find('.delete').trigger('click');
						}
						// if dragged enough to reveal more than half of the options & no options are showing, show options
						else if (draggedEntryOffsetX < -88) {
							$this.removeClass('dragging-x');
							// disable buttons
							$entryDetails.transition({
								x: '-181'
							}, 100, function() {
								$this.addClass('showing-options');
								$this.removeClass('dragging-x');
							});
						}
						// else, slide it back (hide the options)
						else {
							$entryDetails.transition({
								x: '0'
							}, 100, function() {
								$this.removeClass('showing-options');
								$this.removeClass('dragging-x');
							});					
						}
					}
				});

			// }
			

		/*
			>> tap delete entry
		*/		
			
			$('#entries .entry .delete').live('click', function (e) {
				e.preventDefault();
				$entryToDelete = $(this).parent().parent();
				budgetId = $('.budget.active').attr('data-id');
				var entryId = $entryToDelete.attr('data-id');
				// if we're near the bottom (and not at the top) scroll user up a bit before deleting the entry to avoid rough animation
				var top = $('html').offset().top;
				if(nearBottom(84) && top !== 0) {
					$('html, body').animate({
						scrollTop: Math.abs(top + $entryToDelete.height())
					}, 100, function() {
						deleteEntry();
					});
				}
				else {
					deleteEntry();	
				}
				function deleteEntry() {
					$entryToDelete.transition({
						opacity: 0,
						x: '-100%'
					}, 200, function() {
						$entryToDelete.animate({
							height: '0'
						}, 150, function() {
							$entryToDelete.remove();
							// enable buttons
							calculateIncomeAndBalance();
							balance.webdb.deleteEntry(entryId, budgetId);
						});
					});
				}
			});
		

		/*
			>> mark as paid & unpaid
		*/

			$('#entries .option.unpaid').live('click', function (e) {
				e.preventDefault();

				// mark as paid
				var $this = $(this);
				var $entryToMark = $this.parent().parent();
				var $entryDetails = $entryToMark.find('.entry-details');
				var budgetId = parseInt($('#budgets .budget.active').attr('data-id'));
				var entryId = parseInt($entryToMark.attr('data-id'));

				$entryToMark.addClass('paid');
				$this.addClass('paid').removeClass('unpaid');
				$this.find('.icon-unpaid').removeClass('icon-unpaid').addClass('icon-paid');

				// mark as paid in db
				balance.webdb.markAsPaid(budgetId, entryId);

				// hide entry options
				$entryDetails.transition({
					x: '0'
				}, 200, function() {
					$entryToMark.removeClass('showing-options');
					showingOptions = false;
					calculateIncomeAndBalance();

					// show 'mark as paid' popup if not seen before
					if (!localStorage.seenMarkAsPaidPopup) {
					
						nativeAlert(
							'You just marked an item as Paid. Did you know you can toggle between overall Balance and Paid Balance by tapping once on your balance?', 
							null, 
							'Mark as Paid',
							'OK'
						);

						localStorage.seenMarkAsPaidPopup = true;
					}
				});

				
			});

			$('#entries .option.paid').live('click', function (e) {
				e.preventDefault();

				// mark as unpaid
				var $this = $(this);
				var $entryToMark = $this.parent().parent();
				var $entryDetails = $entryToMark.find('.entry-details');
				var budgetId = parseInt($('#budgets .budget.active').attr('data-id'));
				var entryId = parseInt($entryToMark.attr('data-id'));

				$entryToMark.removeClass('paid');
				$this.addClass('unpaid').removeClass('paid');
				$this.find('.icon-paid').removeClass('icon-paid').addClass('icon-unpaid');

				// mark as unpaid in db
				balance.webdb.markAsUnpaid(budgetId, entryId);

				// hide entry options
				$entryDetails.transition({
					x: '0'
				}, 200, function() {
					$entryToMark.removeClass('showing-options');
					showingOptions = false;
					calculateIncomeAndBalance();
				});

			});



					
		/*
			>> star or unstar entry
		*/		
			
			$('#entries .pin').live('click', function (e) {
				e.preventDefault();
				var $this = $(this);
				$entryToPin = $this.parent().parent();
				var $entryDetails = $entryToPin.find('.entry-details');
				
				$entryToPin.addClass('pinned');
				entryToPinId = $entryToPin.attr('data-id');
				balance.webdb.pinEntry(entryToPinId);
				$this.addClass('unpin').removeClass('pin');
				$this.find('.icon-star').removeClass('icon-star').addClass('icon-star-filled');
				$entryDetails.transition({
					x: '0'
				}, 200, function() {
					$entryToPin.removeClass('showing-options');
					showingOptions = false;
				});

			});
			
			$('#entries .unpin').live('click', function (e) {
				e.preventDefault();
				var $this = $(this);
				$entryToUnpin = $this.parent().parent();
				$entryToUnpin.removeClass('pinned');
				entryToUnpinId = $entryToUnpin.attr('data-id');
				currentlyViewedBudgetId = $('.budget.active').attr('data-id');
				balance.webdb.unpinEntry(entryToUnpinId, currentlyViewedBudgetId);
				$this.addClass('pin').removeClass('unpin');
				$this.find('.icon-star-filled').removeClass('icon-star-filled').addClass('icon-star');
				var $entryDetails = $entryToUnpin.find('.entry-details');
				$entryDetails.transition({
					x: '0'
				}, 200, function() {
					$entryToUnpin.removeClass('showing-options');
					showingOptions = false;
				});		
			});
			
			
		/*
			>> tap add budget
		*/
		
			$('#add-budget').on('click', function (e) {
				e.preventDefault();

				// hide any showing budget options
				if ($('#budgets .item.showing-options').length > 0) {
					$('#budgets .item.showing-options .budget-details').transition({
						x: '0'
					}, 100, function() {
						$('#budgets .item.showing-options').removeClass('showing-options');
						addNewBudgetMarkup();
					});
				}
				else {
					addNewBudgetMarkup();	
				}
				
				function addNewBudgetMarkup() {
				
					// add markup
					var budgetMarkup = '<div class="budget item clearfix clear" data-id="">';
					budgetMarkup +=			'<div class="budget-details item-details clearfix theme-border">';
					budgetMarkup +=				'<textarea class="budget-title item-title theme-color" rows="1" cols="20" placeholder="E.g. March" autocapitalize="on"></textarea>';
					budgetMarkup +=				'<p class="budget-title item-title fake hidden theme-color"></p>';
					budgetMarkup +=				'<i class="icon-arrow-right"></i> ';
					budgetMarkup +=			'</div>';
					budgetMarkup +=			'<div class="budget-options item-options theme-bg theme-border">';
					budgetMarkup +=				'<div class="delete option">'; 
					budgetMarkup +=					'<div class="icon">';
					budgetMarkup +=						'<i class="icon-bin"></i>';
					budgetMarkup +=						'<span>Delete</span>';
					budgetMarkup +=					'</div>';
					budgetMarkup +=				'</div>';
					budgetMarkup +=				'<div class="rename option">';
					budgetMarkup +=					'<div class="icon">';
					budgetMarkup +=						'<i class="icon-edit"></i>';
					budgetMarkup +=						'<span>Rename</span>';
					budgetMarkup +=					'</div>';
					budgetMarkup +=				'</div>';
					budgetMarkup +=				'<div class="export option disabled-on-free">';
					budgetMarkup +=					'<div class="icon">';
					budgetMarkup +=						'<i class="icon-export"></i>';
					budgetMarkup +=						'<span>Export</span>';
					budgetMarkup +=					'</div>';
					budgetMarkup +=				'</div>';					
					budgetMarkup +=			'</div>';
					budgetMarkup +=		'</div>';
					var $budgetMarkup = $(budgetMarkup);
					$budgetMarkup.appendTo('#budgets');
					
					// select textarea and setup autogrow
					textarea = $budgetMarkup.find('textarea.budget-title');
					textarea.select();

					// textarea.focus();
					initializeAutogrowTextarea(textarea);
					
					var orderNumber = parseInt($('#budgets .budget').length, 10) + 1;
					// add in database
					balance.webdb.addBudget(orderNumber, $budgetMarkup);

					// android - scroll down to bottom
					if (platform === 'Android') {
						setTimeout(function(){
							scrollToBottom();
						}, 400);
					}

				}
					
			});			
		
		/*
			>> tap delete budget
		*/
			
			$('#budgets .budget .delete').live('click', function (e) {
				e.preventDefault();
				$budgetToDelete = $(this).parent().parent();
				budgetId = $budgetToDelete.attr('data-id');
				// if we're near the bottom (but not at the top), scroll the user up a bit first
				var top = $('html').offset().top;
					
				if(nearBottom(42) && top !== 0 && platform === 'iOS') {
					$('html, body').animate({
						scrollTop: Math.abs(top + $budgetToDelete.height())
					}, 100, function() {
						deleteBudget();
					});
				}
				else {
					deleteBudget();	
				}
				
				function deleteBudget() {

					// confirm deletion
					nativeConfirm(
						'Are you sure you want to delete this Budget?',	// message
						confirmDelete,	                                // callback (with tapped button index as parameter)
						'Delete Budget?',	                            // title
						['Delete','Cancel'],	                        // button labels
						null,	                                        // fallback message 
						1,	                                            // fallback - ok pressed
						2	                                            // fallback - cancel pressed
					);

					// confirm deletion callback
					function confirmDelete(buttonIndex) {
						// delete pressed
						if (buttonIndex === 1) {
							setTimeout(function(){
								doDelete();
							}, 200);
						}
						// cancel pressed
						else {
							$budgetToDelete.find('.budget-details').transition({
								x: '0%'
							}, 200, function() {
								$budgetToDelete.removeClass('swipe-to-delete showing-options');
							});
						}
					}

					// do the delete
					function doDelete() {
						$budgetToDelete.transition({ 
							opacity: 0,
							x: '-100%'
						}, 200, function() {
							$budgetToDelete.animate({
								height: '0'
							}, 100, function() {
								$budgetToDelete.remove();
								// enable buttons
								balance.webdb.deleteBudget(budgetId);
							});
						});
					}
				}
			});			
			
			
		/*
			>> tap rename budget
		*/
		
			$('#budgets .budget .rename').live('click', function (e) {
				e.preventDefault();
				e.stopPropagation();
				var $this = $(this);
				$budgetToRename = $this.parent().parent();
				$budgetDetails = $budgetToRename.find('.budget-details');

				$textarea = $budgetToRename.find('textarea.budget-title');
				$fakeTextarea = $budgetToRename.find('p.budget-title').addClass('hidden');
				$textarea.removeClass('off-page').select();
				
				$budgetToRename.addClass('editing');
				$budgetDetails.css({x: 0});
				$budgetToRename.removeClass('showing-options');

				$houseAdIframe.addClass('hidden');

			});
			
			$('#budgets .budget:not(.editing) textarea.budget-title').live('touchstart', function (e) {
				e.preventDefault();
				$(this).blur();
			});
			
			$('#budgets .budget textarea.budget-title').live('blur', function (e) {
				var $this = $(this);
				var $budgetTitle = $this.val();
				$budget = $this.parent().parent();
				
				// remove focus class from budget
				$budget.removeClass('focus-all');
				
				// show pro upgrade banner
				$proUpgradeBanner.removeClass('off-bottom');
				
				// show title bar
		    	setTimeout(function(){
		    		if ($('#budgets-container .focus-left, #budgets-container .focus-right, #budgets-container .focus-all').length === 0) {
				    	// change position of title bar back to fixed
				    	$('#budgets-container .title-bar').removeClass('pos-absolute');
				    	$('#budgets').removeClass('padding-top-50');
				    }
		    	}, 100);

				$budget.removeClass('editing');
				$this.addClass('off-page');
				// update fake textarea
				$budget.find('p.budget-title').text($budgetTitle).removeClass('hidden');

				$houseAdIframe.removeClass('hidden');
			});
			
			$('#budgets .budget textarea.budget-title').live('focus', function (e) {
				var $this = $(this);
				$budget = $this.parent().parent();
				
				// add focus class to entry
				$this.parent().parent().addClass('focus-all');
				
				// hide pro upgrade banner
				$proUpgradeBanner.addClass('off-bottom');
				
				// change title bar position to absolute 
				$('#budgets-container .title-bar').addClass('pos-absolute');
				$('#budgets').addClass('padding-top-50');
				
				$budget.addClass('editing');

				$houseAdIframe.addClass('hidden');
			});


		/*
			>> tap export budget
		*/
		
			$('.export').live('click', function (e) {
				e.preventDefault();
				e.stopPropagation();
				var $this = $(this);
				if (freeVersion) {
					$('#export-popup').show().transition( {y: '0%'}, 300, function() {
						animating = false;
					});
				}
				else {
					// if tapped on the budgets page
					if ($this.hasClass('option')) {
						var budgetToExportId = $this.parents('.budget').attr('data-id');
					}
					// else if tapped on the entries page option menu
					else {
						var budgetToExportId = $('#budgets .budget.active').attr('data-id');

					}
					// export budget
					balance.webdb.getAllEntries(budgetToExportId, exportBudget);
				}
			});
		
			
		/*
			>> blur budget (update)
		*/
			
			$('#budgets .budget-title').live('blur', function (e) {
				e.preventDefault();
				var $this = $(this);
				var $budgetBlurred = $this.parent().parent();
				var title = $(this).val();
				var checkForBudgetId;
				var budgetId;

				$budgetBlurred.removeClass('editing');

				$houseAdIframe.removeClass('hidden');

				// if data-id attribute is empty (because we're still waiting for budget to be added to db - and attribute to be updated) then wait for a data-id attribute
				if ($budgetBlurred.attr('data-id') === '') {
					// console.log('NO FUCKING BUDGET ID!');
					var checkForBudgetIdCount = 0;
					checkForBudgetId = setInterval(function(){
						checkForBudgetIdCount++;
						if (checkForBudgetIdCount === 50) {
							clearInterval(checkForBudgetId);
						}
						else if ($budgetBlurred.attr('data-id') !== '') {
							clearInterval(checkForBudgetId);
							// console.log('BUDGET ID FUCKING ARRIVED, CONTINUE!');
							budgetId = $budgetBlurred.attr('data-id');
							finishUpdatingBudget();
						}
					}, 100);
				}
				else {
					budgetId = $budgetBlurred.attr('data-id');
					finishUpdatingBudget();
				}

				function finishUpdatingBudget() {
					// CHEAT - enable/disable pro version
					if (title === '+Pro') {
						enableProVersion();
					}
					if (title === '-Pro') {
						disableProVersion();
					}
					// update budget in database
					balance.webdb.editBudget(budgetId, title);

					// if android, refresh banner
					if (platform === 'Android') {
						refreshBanner();
					}
				}
			});
		

		/*
			>> toggle total income or expenses
		*/

			var $totalArrowBox = $('.total-income-expenses .arrow_box')

			var infoToggleSwitchEvent = (platform == 'Browser' ? 'click' : 'touchstart')

			// show expenses
			$('#total-income-container').on(infoToggleSwitchEvent, function (e) {
				showExpenses();
			});

			// show income
			$('#total-expenses-container').on(infoToggleSwitchEvent, function (e) {
				showIncome();
			});

			// load preference on session start
			if (localStorage.toggleIncomeExpenses == 'expenses') {
				showExpenses();
			}

			// click toggle arrow box prompt to hide it forever
			$totalArrowBox.click(function() {
				$totalArrowBox.remove();
				localStorage.seenTotalArrowBox = true;
			});

			// hide total arrow box prompt if already seen
			if (localStorage.seenTotalArrowBox) {
				$totalArrowBox.remove();
			}

			// fn: show expenses
			function showExpenses() {
				$('#total-income-container').addClass('hidden');
				$('#total-expenses-container').removeClass('hidden');
				localStorage.toggleIncomeExpenses = 'expenses';
				$totalArrowBox.trigger('click');
			}

			// fn: show income
			function showIncome() {
				$('#total-expenses-container').addClass('hidden');
				$('#total-income-container').removeClass('hidden');
				localStorage.toggleIncomeExpenses = 'income';
				$totalArrowBox.trigger('click');
			}


		/*
			>> toggle balance or paid
		*/

			var $balanceArrowBox = $('.balances .arrow_box')

			$('#remaining-balance-container').on(infoToggleSwitchEvent, function (e) {
				showPaid();
			});

			$('#remaining-paid-container').on(infoToggleSwitchEvent, function (e) {
				showBalance();
			});

			// load preference on session start
			if (localStorage.toggleBalancePaid == 'paid') {
				showPaid();
			}

			// click toggle arrow box prompt to hide it forever
			$balanceArrowBox.click(function() {
				$balanceArrowBox.remove();
				localStorage.seenBalanceArrowBox = true;
			});

			// hide total arrow box prompt if already seen
			if (localStorage.seenBalanceArrowBox) {
				$balanceArrowBox.remove();
			}

			// fn: show paid
			function showPaid() {
				$('#remaining-balance-container').addClass('hidden');
				$('#remaining-paid-container').removeClass('hidden');
				localStorage.toggleBalancePaid = 'paid';
				$balanceArrowBox.trigger('click');
			}

			// fn: show balance
			function showBalance() {
				$('#remaining-paid-container').addClass('hidden');
				$('#remaining-balance-container').removeClass('hidden');
				localStorage.toggleBalancePaid = 'balance';
				$balanceArrowBox.trigger('click');
			}


		/*
			>> disable unwanted key presses
		*/
		
			/* make enter key on budget title and entry title blur the field */
			$('#budgets textarea.budget-title, #entries textarea.entry-title').live('keydown', function (e){
				var $this = $(this);

				// if enter pressed, blur field
				if (e.keyCode === 13) {
					document.activeElement.blur();
					$this.blur();
				}
			});
			
			/* 
				[1] disable all except numbers, decimal point, backspace, and delete on number fields
			   	[2] disable 2nd decimal points 
			*/
			
				$('#entries input.amount').live('keypress', function (e){
					var inputValue = $(this).val();
					
					var keyPressed = String.fromCharCode( e.which );
					var allowedKeyPresses = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.'];
					
					// if enter pressed, blur field
					if (e.keyCode === 13) {
						document.activeElement.blur();
						$this.blur();
					}

					// if a disallowed key was pressed, return false
					if ($.inArray( keyPressed, allowedKeyPresses ) === -1) {
						e.preventDefault();
					}
					// else, if '.' was pressed, return false if there is already a '.' in the field
					else {
						if ((keyPressed === '.' && inputValue.indexOf(keyPressed) >= 0) || keyPressed === '.' && inputValue === '') {
							e.preventDefault();
						}
					}
				});			

		/*
			>> tutorial
		*/
		
			/*
				>>> show or hide tutorial
			*/
				
				// show tutorial
				$('#show-tutorial').on('click', function (e) {
					e.preventDefault();
					// if nothing is animating
					if (animating === false) {
						animating = true;
						$('#tutorial').show().transition( {y: '0%'}, 300, function() {
							var lastPage = localStorage.getItem('lastPage');
							if (lastPage === 'budgets') {
								$('#budgets-container').addClass('hidden');
							}
							else if (lastPage === 'entries') {
								$('#entries-container').addClass('hidden');
							}
							animating = false;
						});
						stopTutorialAnimation[currentNav] = false;
						tutorialAnimation(currentNav);
						carousel.init();
					}
					
				});
				
				// hide tutorial
				$('#tutorial-close').on('click', function (e) {
					e.preventDefault();
					// unlock scrolling (show entries / budgets)
					var lastPage = localStorage.getItem('lastPage');
					if (lastPage === 'budgets') {
						$('#budgets-container').removeClass('hidden');
					}
					else if (lastPage === 'entries') {
						$('#entries-container').removeClass('hidden');
					}
					$('#tutorial').transition( {y: '100%'}, 300, function() {
						$(this).hide();
					});
					stopTutorialAnimation[currentNav] = true;
					tutorialAnimation(currentNav, 'reset');
					localStorage.firstRunTutorialDisplayed = true;
				});
				
				
		/*
			>> popups
		*/
		
				
			/*
				>>> pro upgrade popup
			*/
			
				// show pro upgrade popup
				$('.show-pro-upgrade-popup').on('click', function (e) {
					e.preventDefault();
					// ga - track event
					if ($(this).hasClass('pro-upgrade-banner')) {
						gaTrackEvent('Pro Upgrade Banner', 'Pro Upgrade Banner > Tapped Pro Upgrade Banner', null);
					}
					else {
						gaTrackEvent('Settings', 'Settings > Fudget Pro Upgrade', null);
					}
					if (freeVersion) {
						// if nothing is animating
						if (animating === false) {
							animating = true;
							// hide any pro upgrade feature popups
							$('.pro-upgrade-feature-popup').transition({y: '100%'}, 300, function() {
								$(this).hide();
							});
							// show pro upgrade popup
							$('#pro-upgrade-popup').show().transition( {y: '0%'}, 300, function() {
								animating = false;
							});
						} 
					}
					else {
						showAlreadyUpgradedPopup();
					}
				});
		 
		 		// close pro upgrade popup
		 		$('#pro-upgrade-popup .popup-close').click(function (e) {
		 			e.preventDefault();
			 		$('#pro-upgrade-popup').transition( {y: '100%'}, 200, function() {
			 			$(this).hide();
			 		});
		 		});

		 		// mark as paid popup
		 		$('.show-dropbox-save-and-restore-popup').click(function (e) {
		 			e.preventDefault();
		 			$('#dropbox-save-and-restore-popup').show().transition( {y: '0%'}, 300, function() {
		 				animating = false;
		 			});
		 		});
		 		
		 		// no-ads-themes popup
		 		$('.show-no-ads-themes-popup').click(function (e) {
		 			e.preventDefault();
		 			$('#no-ads-themes-popup').show().transition( {y: '0%'}, 300, function() {
		 				animating = false;
		 			});
		 		});
		 		
		 		// calculator popup
		 		$('.show-calculator-popup').click(function (e) {
		 			e.preventDefault();
		 			$('#calculator-popup').show().transition( {y: '0%'}, 300, function() {
		 				animating = false;
		 			});
		 		});
		 		
		 		// export popup
		 		$('.show-export-popup').click(function (e) {
		 			e.preventDefault();
		 			$('#export-popup').show().transition( {y: '0%'}, 300, function() {
		 				animating = false;
		 			});
		 		});

		 		// close pro upgrade feature popup
		 		$('.pro-upgrade-feature-popup .popup-close').click(function (e) {
		 			$(this).parent().transition({y: '100%'}, 300, function() {
		 				$(this).hide();
		 			});
		 		});

 				// upgrade popup - click try pro
 				$('.try-pro').click(function (e) {
 		       		if (!$(this).hasClass('disabled')) {

 		       			// variables
 		       			var message = 'Check out this app Fudget I\'ve been using to manage my money!';
 		       			var subject = 'Fudget for iOS and Android. ';
 		       			var imageUrl = 'http://fudget.dconnell.co.uk/assets/images/fudget-action-shot.png';

 		       			gaTrackEvent('Upgrade Popup', 'Upgrade Popup > Try Pro', null);

 		       			// confirm share to start pro trial
 		       			nativeConfirm(
 		       				'To unlock Fudget Pro for 14 days, simply share this app on Facebook or Twitter',	// message
 		       				tryProConfirmCallback,					                                            // callback
 		       				'Try Fudget Pro!',						                                            // title
 		       				['Share on Facebook', 'Share on Twitter', 'Close'],							        // button labels
 		       				null,									                                            // fallback message
 		       				4										                                            // fallback - ok pressed
 		       			);

 		       			// confirm share callback
 		       			function tryProConfirmCallback(buttonIndex) {
		       		     	// Share on Facebook & try pro pressed
		       		     	if (buttonIndex == 1) {
		       		     		// share on facebook
					     		window.plugins.socialsharing.shareViaFacebook(
					     			'Fudget - fast, simple money management',
					     			null,
					     			appWebsiteUrl, // url
					     			function() {
					     				startProTrial();
					     				gaTrackEvent('Upgrade Popup', 'Upgrade Popup > Try Pro > Shared on Facebook and Tried Pro', null);
					     			}, function(errormsg){
					     			},
					     			null
					     		);	
		       		     	}
		       		     	// Share on Twitter & try pro pressed
		       		     	if (buttonIndex == 2) {
					     		// share on twitter
						     	window.plugins.socialsharing.shareViaTwitter(
						     		message + ' ' + appWebsiteUrl, 
						     		imageUrl /* img */, 
						     		null,
						     		function() {
						     			startProTrial();
					     				gaTrackEvent('Upgrade Popup', 'Upgrade Popup > Try Pro > Shared on Twitter and Tried Pro', null);
						     		}, 
						     		function(errormsg){
						     		},
						     		null
						     	);
		       		     	}
		       		     	// browser fallback
		       		     	if (buttonIndex === 4) {
		       		     		startProTrial();
		       		     	}
	       		     		function startProTrial() {
		       		     		// unlock pro version
		       		     		enableProVersion();
		       		     		// add pro trial expiry date to localStorage
		       		     		var currentDate = new Date().getTime();
		       		     		// add 14 days to expiry date    
		       		     		proTrialExpiryDate = currentDate + (14*24*60*60*1000); // temp - add 14 days (in milliseconds)
		       		     		// proTrialExpiryDate = currentDate + (60*2000); // temp - add 1 minute (in milliseconds)
		       		     		// add expiry date to localStorage
		       		     		localStorage['proTrialExpiryDate'] = proTrialExpiryDate;

		       		     		// alert user that they are now on pro version
								nativeAlert(
									'Thanks for the share! You have 14 days to try Fudget Pro!',
									null,
									'Thanks!',
									'OK'
								);
	       		     		}
 		       			}


 		       		}				
 				});
		 		

		/*
			>> in-app purchases
		*/

			var iapId = 'fudget.pro.upgrade',
			// var iapId = 'android.test.purchased',
				iapAlias = 'Fudget Pro Upgrade',
				anyBuyButtonPressed = false;

			// real in-app purchases
			if (isNative && !demoMode) {

				function initializeStore() {

					// if there's an internet connection
					if(navigator.network.connection.type !== Connection.NONE){

						var userTappedRestore = false;

						if (!window.store) {
						    /// log('Store not available');
						    return;
						}
						else {
							/// log('Store avaiable');
						}

					    // Let's set a pretty high verbosity level, so that we see a lot of stuff
					    // in the console (reassuring us that something is happening).
					    store.verbosity = store.INFO;

					    // register pro upgrade iap
					    store.register({
					        id:    iapId,
					        alias: iapAlias,
					        type:  store.NON_CONSUMABLE
					    });

					    // When any product gets updated, refresh the HTML.
					    store.when("product").updated(function (p) {
					        /// log('refresh IAPs');
					    });

					    // when fudget.pro.upgrade is updated, add price to button
					    // product attributes - product.owned, loaded, valid, alias, title, description, id, price
					    store.when(iapId).updated(function (product) {
					    	/// log(product.price);
					    	$('.pro-upgrade-price').text(product.price);
					    });

					    // when purchase of fudget.pro.upgrade is approved, enable pro version
					    store.when(iapId).approved(function (order) {
					        // finish order
					        order.finish();
					        /// log('approved');
					    });

					    // finished transaction of fudget.pro.upgrade
					    store.when(iapId).finished(function (order) {
					    	// thank the user if they've not already upgraded
					    	if (!localStorage['IAPfudgetProUpgrade']) {
					    		// if user is restoring purchases, show generic message
					    		if (userTappedRestore) {
					    			nativeAlert(
					    				'Thanks! Your purchases have been restored!',
					    				null,
					    				'Thanks'
					    			);
					    		}
					    		// else show specific message
					    		else {
					    			showAlreadyUpgradedPopup();
					    		}
					    	}
					    	// enable pro version
					    	enableProVersion();
					    	// GA - track event
					    	gaTrackEvent('Get Pro Upgrade', 'Get Pro Upgrade > Purchase successful', null)
					    });


					    // report all errors
					    store.error(function(error) {

					    	// only show error message and log error if any buy button was pressed

					    	if (anyBuyButtonPressed) {
						        nativeAlert(error.message, null, 'Error');
						        // GA - track event
						        gaTrackEvent('Get Pro Upgrade', 'Get Pro Upgrade > Error > ' + error.message, null)
					    	}

					    });

					    // When every goes as expected, it's time to celebrate!
					    // The "ready" event should be welcomed with music and fireworks,
					    // go ask your boss about it! (just in case)
					    store.ready(function() {
					       	/// log("\\o/ STORE READY \\o/");

					       	// enable store related buttons
					       	$('.get-pro-upgrade, .restore-purchases').removeClass('disabled').unbind('click');

					       	// click purchase upgrade button
					       	$('.get-pro-upgrade').click(function (e) {
					       		// GA - track event
					       		gaTrackEvent('Get Pro Upgrade', 'Get Pro Upgrade > Get Pro Upgrade Button tapped', null)

					       		anyBuyButtonPressed = true;
					       		var $purchaseUpgradeButton = $(this);
					       		e.preventDefault();
					       		
					       		if (!$purchaseUpgradeButton.hasClass('disabled')) {
					       			// order fudget.pro.upgrade IAP
					       			/// log('order');
						       		store.order(iapId);
						       		// store.refresh();
					       		}
					       		else {
					       			store.refresh();
					       		}
					       	});

					       	// enable restore purchases button
					       	$('.restore-purchases').click(function (e) {
					       		anyBuyButtonPressed = true;
					       		var $restorePurchasesButton = $(this);
					       		e.preventDefault();
					       		if (!$restorePurchasesButton.hasClass('disabled')) {
					       			userTappedRestore = true;
					       			store.refresh();
					       			if (platform === 'Android') {
					       				store.order(iapId);
					       			}
					       		}
					       		else {
					       			store.refresh();
					       		}
					       	});


					    });

					    // After we've done our setup, we tell the store to do
					    // it's first refresh. Nothing will happen if we do not call store.refresh()
					    store.refresh();

					}

				}

				initializeStore();

				// if upgrade or restore buttons are disabled and clicked, prompt the user to check their internet connection
				$('.get-pro-upgrade, .restore-purchases').bind('click', function (e) {
					if ($(this).hasClass('disabled')) {
						nativeAlert(
							'Sorry, could not connect to the Store. Please check your internet connection and try again.',
							null,
							'Can\'t connect to store'
						);
						initializeStore();
					}
				});

			}
			// dummy in-app purchases for web app / demo mode
			else{
		       	// click purchase upgrade button
		       	$('.get-pro-upgrade').click(function (e) {
		       		e.preventDefault();

	       			nativeConfirm(
	       				'Do you want to buy one Fudget Upgrade for £2.99?',	// message
	       				getProUpgradeCallback,					            // callback
	       				'Confirm Your In-App Purchase',						// title
	       				['Cancel', 'Buy'],							        // button labels
	       				null,									            // fallback message
	       				2										            // fallback - ok pressed
	       			);

	       			function getProUpgradeCallback(buttonIndex) {
	       		     	// buy pressed
	       		     	if (buttonIndex === 2) {
	       		     		enableProVersion();
	       		     		showAlreadyUpgradedPopup();
	       		     	}
	       			}

		       	});
			}
			


		/* 
			>> tap title bar - scroll to top
		*/

			$('#budgets-container>.title-bar, #entries-container>.title-bar, #settings-container>.title-bar').click(function (e) {
				$('html, body').animate({
					scrollTop: 0
				}, 100);
			});


		/*
			>> show options menu
		*/

			$('.show-menu').click(function (e) {
				e.preventDefault();
				e.stopPropagation();
				if (!$('.menu').hasClass('show')) {
					$('.menu').removeClass('hidden');
					setTimeout(function(){
						$('.menu').addClass('show');
					}, 100);
					// update text
					$('.show-menu small').text('Close');
				}
				else {
					$('.menu').removeClass('show');
					setTimeout(function(){
						$('.menu').addClass('hidden');
					}, 300);
					// update text
					$('.show-menu small').text('Options');
				}
			});

			// click out
			$('body').click(function (e) {
				if ($('.menu').hasClass('show')) {
					$('.menu').removeClass('show').addClass('hidden');
					// update text
					$('.show-menu small').text('Options');
				}
			});


		/*
			>> join newsletter
		*/

			$('.join-newsletter-form').submit(function (e) {

				e.preventDefault();

				// get fields
				var name = $('.newsletter-name').val();
				var email = $('.newsletter-email').val();
				var list;
				var url = 'http://sendy.dconnell.co.uk/subscribe';
				var errorMessage = 'Couldn\'t connect. Please check your internet connection and try again.';
				if (platform === 'Android') {
					// list = 'C1I8Oe0kT5TJLkiS7UhCSA'; // Test list
					list = '2eR84aJ17634qaF6ScKH1NpQ'; // Android list
				}
				else {
					// list = 'C1I8Oe0kT5TJLkiS7UhCSA'; // Test list
					list = 'NfLWde5AFpQSRj3x892plRsQ'; // iOS list
				}
				var data = { name: name, email: email, list: list  };

				// send request
				$.ajax({
					type: "POST",
					url: url,
					data: data,
					success: function(data){
						console.log('data: ', data);
				  		var message = $(data).find('h2').text();
				  		console.log('message: ' + message);
				  		if (message === "You're subscribed!" || data === "") {
				  			message = "You're subscribed!";
				  			var alertHeading = 'Thanks!';
				  			// clear form
				  			$('.join-newsletter-form input').val('').blur();
				  		}
				  		else {
				  			var alertHeading = 'Sorry!';
				  		}
				  		// GA - track feedback - prompt - show prompt
						gaTrackEvent('Newsletter', 'Newsletter (' + platform + ') > ' + message, null);
						// inform the user of the result
				  		nativeAlert(
				  			message,
				  			null,
				  			alertHeading
				  		);
					},
					error: function(XMLHttpRequest, textStatus, errorThrown) {
			  	  		// GA - track feedback - prompt - show prompt
			  			gaTrackEvent('Newsletter', 'Newsletter (' + platform + ') > Error', null);

						// inform the user of the result
				  		nativeAlert(
				  			errorMessage,
				  			null,
				  			'Sorry!'
				  		);
					}
				});

			});


		/*
			>> charts
		*/

			var chart;
			var total;

			$('.show-charts-popup').click(function (e) {
				e.preventDefault();
				$('#charts-popup').show().transition( {y: '0%'}, 300, function() {
					animating = false;

					// get the raw data
					var rawData = [];
					$('#entries .entry.expense').each(function(index) {
						var $this = $(this);
						var thisLabel = $this.find('textarea.entry-title').val();
						var thisValue = $this.find('input.amount').val();
						// newly created entries have no amount value, so set these to 0 to avoid NaN bugs later
						if (thisValue === '') {
							thisValue = 0;
						}
						thisValue = parseFloat(thisValue);
 					    rawData.push({
							label: thisLabel,
							value: thisValue
					    });
					});

					// generate some nice colors
					var colors = randomColor({count: rawData.length + 1});

					// generate the chart from raw data
					generateChart(rawData, 'pie');

					// function to generate the chart
					function generateChart(rawData, chartType) {
						// sort the data ascending by value
						rawData.sort(ascending);

						// loop through raw data and create data array for chart
						var data = [];
						$.each( rawData, function(i) {
							var trimmedLabel = this.label;
							if (trimmedLabel.length > 20) {
								trimmedLabel = trimmedLabel.substring(0,20) + '..';
							}
							data.push({
								value: this.value,
								color: colors[i],
								label: trimmedLabel
							});
						});
						
						// get the total value of all items
						total = 0;
						$.each( data, function() {
							total = total + this.value
						});

						// close the popup if total is 0 and warn user
						if (total === 0) {
							nativeAlert(
								'Please enter a value for at least one expense.',
								function() {
									$('.charts-popup .popup-close').trigger('click');
								},
								'Error!'
							);
						}
						
						// get the cut off point in percent (below which items will be grouped into 'other')
						var cutOff = total * 0.03; // 3 percent
						
						// look at each item value starting with the lowest, find out which index is the cut off
						var runningTotal = 0;
						var cutOffIndex;
						$.each( data, function(index) {
							runningTotal = runningTotal + this.value;
							// console.log(index + ': ' + runningTotal);
							if (runningTotal > cutOff) {
								cutOffIndex = index - 1;
								return false;
							}
						});

						// if there's more than 1 item to bundle into 'other'
						if (cutOffIndex > 0) {
							// calculate the 'other' total
							var otherTotal = 0;
							for (i = 0; i <= cutOffIndex; i++) {
								otherTotal += data[i].value;
							}
							
							// remove the cut-off items from the array
							data.splice(0, cutOffIndex + 1);
						}
					 
						// sort data descending before rendering
						data.sort(descending);
						
						// insert the 'other' item
						if (cutOffIndex > 0) {
							data.push({
								value: otherTotal,
								color: colors[colors.length - 1],
								label: 'Other'
							});
						}
						
						// chart options
						if (platform === 'Android') {
							var chartAnimation = false;
						}
						else {
							var chartAnimation = true;
						}
						var options = {
								segmentShowStroke: false,
								animateRotate: chartAnimation,
								animateScale: false,
								animation: chartAnimation,
								//percentageInnerCutout: 50,
								tooltipTemplate: function(v) {
										return v.label + ' - ' + currencySymbol + v.value.toFixed(2)},
								legendTemplate : "<ul class=\"<%=name.toLowerCase()%>-legend\"><% for (var i=0; i<segments.length; i++){%><li><span class=\"color\" style=\"background-color:<%=segments[i].fillColor%>\"></span><%if(segments[i].label){%><%=segments[i].label%><%}%> - <%=currencySymbol%><%=segments[i].value.toFixed(2)%> - <strong><%=((segments[i].value/total)*100).toFixed(2)%>%</strong><span class=\"remove-data-item touchable\" data-index='<%=i%>'>&times;</span></li><%}%></ul>"
						}

						var ctx = document.getElementById("chart").getContext("2d");
						
						if (chartType === 'pie') {
							// generate chart
							chart = new Chart(ctx).Pie(data, options);
							
							// generate legend
							document.getElementById('js-legend').innerHTML = chart.generateLegend();
						}

						// remove a data item on X click
						$('#js-legend').on('click', '.remove-data-item', function (e) {
							index = $(this).data('index');
							chart.removeData(index);
							if (platform === 'Android') {
								chart.render();
							}
							document.getElementById('js-legend').innerHTML = chart.generateLegend();	
						});
						
					}

					/* functions */

					function getRandomColor() {
							var letters = '0123456789ABCDEF'.split('');
							var color = '#';
							for (var i = 0; i < 6; i++ ) {
									color += letters[Math.floor(Math.random() * 16)];
							}
							return color;
					}

					function descending(a,b) {
						if (a.value > b.value)
							return -1;
						else if (a.value < b.value)
							return 1;
						else 
							return 0;
					}

					function ascending(a,b) {
						if (a.value < b.value)
							return -1;
						else if (a.value > b.value)
							return 1;
						else 
							return 0;
					}					

				});
			});

			// close charts popup
			$('.charts-popup .popup-close').click(function (e) {
				$(this).parent().transition({y: '100%'}, 300, function() {
					$(this).hide();
					// destroy chart
					chart.destroy();
					// clear legend
					$('#js-legend').html('');
					// clear 'remove' click events
					$('#js-legend').off('click', '.remove-data-item');
				});
			});


		/*
			>> toggle entry date column
		*/

			// variables
			var $dateColumnToggle = $('.date-column-toggle');

			// show date column on load (if enabled)
			if (localStorage.showDateColumn === 'true') {
				showDateColumn();
			}

			// toggle on settings page
			$('#show-date-column').change(function (e) {
				// show date column
				if ($(this).is(':checked')) {
					showDateColumn();
				}
				// hide date column
				else {
					hideDateColumn();
				}
			});

			// toggle on menu
			$dateColumnToggle.click(function (e) {
				e.preventDefault();
				var $this = $(this);
				// show date column
				if (!$this.hasClass('on')) {
					showDateColumn();
				}
				// hide date column
				else {
					hideDateColumn();
				}
			});

			// show date column
			function showDateColumn() {
				$dateColumnToggle.addClass('on');
				$('#show-date-column').attr('checked', 'checked');
				$('#entries').addClass('show-date-column');
				localStorage.showDateColumn = true;
			}

			// hide date column
			function hideDateColumn() {
				$dateColumnToggle.removeClass('on');
				$('#show-date-column').removeAttr('checked');
				$('#entries').removeClass('show-date-column');
				localStorage.showDateColumn = false;
			}


		/*
			>> tap show review icon
		*/

			$('.show-feedback-module').click(function (e) {
				e.preventDefault();
				e.stopPropagation();
				// GA - track feedback - title bar - tapped review icon
				gaTrackEvent('Title bar', 'Title bar > Tapped Review icon', null);
				promptForFeedback();
			});


		/*
			>> android back button
		*/

			document.addEventListener("backbutton", function() {
				// if chart showing, close it
				if ($('#charts-popup').is(":visible")) {
					$('.charts-popup .popup-close').trigger('click');
				}
				// if calculator showing, close it
				else if ($('.calculator-container').is(":visible")) {
					$('.calculator-container .close-calculator').trigger('click');
				}
				// if on entries page...
				else if (localStorage.lastPage === 'entries') {
					// if there's a history, go to budgets page
					if (hasHistory) {
						showBudgetsList();
					}
					// otherwise, user never left this page, quit app
					else {
						navigator.app.exitApp();
					}
				}
				// if on budgets page, quit
				else if (localStorage.lastPage === 'budgets') {
					navigator.app.exitApp();
				}
				// if on settings page, go back to budgets
				else if (localStorage.lastPage === 'settings') {
					$('#settings-container').trigger('swipeleft');
				}
			}, false);


		/*
			>> dropbox save & restore setup
		*/

			var $dropboxSaveAndRestore = $('#dropbox-save-and-restore');

			// setup switch on load
			if (localStorage.getItem('dropboxSaveAndRestore') === 'true') {
				$dropboxSaveAndRestore.attr('checked', 'checked');
				// show dropbox options
				$('.dropbox-options').removeClass('hidden');
			}

			// user changes dropbox save & restore switch
			$dropboxSaveAndRestore.change(function (e) {

				e.preventDefault();
				var $this = $(this);

				// switch is on
				if ($this.is(':checked')) {

					// if free version, show popup
					if (freeVersion) {

						animating = true;
						$('#dropbox-save-and-restore-popup').show().transition( {y: '0%'}, 300, function() {
							animating = false;
							// disable Dropbox Save & Restore option
							$dropboxSaveAndRestore.removeAttr('checked');
						});

					}
					// otherwise enable/start authentication
					else {

						// if user has already authenticated with dropbox, switch on dropbox save & restore
						if (localStorage.access_token && localStorage.access_token !== '') {
							console.log('user has already authenticated');
							// turn on dropbox save & restore
							localStorage.dropboxSaveAndRestore = true;
		                    // show dropbox options
		                    $('.dropbox-options').removeClass('hidden');
		                    // check for new data
		                    balance.webdb.checkIfDropboxDataNewer();
						}
						// otherwise, authenticate dropbox
						else {
							authenticateDropbox();
						}
						
					}

				}
				// switch is off
				else {
					// turn off dropbox save & restore
					localStorage.dropboxSaveAndRestore = false;
					// hide dropbox options
					$('.dropbox-options').addClass('hidden');
				}
			});

			// fn: authenticate dropbox
			function authenticateDropbox() {
				var access_token;

				// url of my dropbox redirect page (dropbox will send users to this after authentication)
				var dropboxRedirectUrl = 'https://dropbox.dconnell.co.uk';

				// open dropbox authentication page
				var win = window.open( 'https://www.dropbox.com/1/oauth2/authorize?client_id=ub85ketkmpwfzck&response_type=token&disable_signup=true&redirect_uri=' + dropboxRedirectUrl, '_blank', 'location=no,toolbar=yes');

				// Once the InAppBrowser finishes loading
				win.addEventListener( "loadstop", function() {

				    // Clear out the access_token in localStorage for subsequent opens.
				    win.executeScript({ code: "localStorage.setItem( 'access_token', '' );" });

				    // Start an interval
				    var loop = setInterval(function() {

				        // Execute JavaScript to check for the existence of an access_token in the
				        // child browser's localStorage.
				        win.executeScript(
				            {
				                code: "localStorage.getItem( 'access_token' )"
				            },
				            function( values ) {
				                access_token = values[ 0 ];

				                // If a access_token was set, clear the interval and close the InAppBrowser.
				                if ( access_token ) {
				                    clearInterval( loop );
				                    win.close();

				                    localStorage.access_token = access_token;
				                    localStorage.dropboxSaveAndRestore = true;

				                    // show dropbox options
				                    $('.dropbox-options').removeClass('hidden');

				                    // confirm dropbox save and restore active
				                    setTimeout(function(){
					                    confirmDropboxActive();
				                    }, 1000);

				                }
				            }
				        );
				    }, 1500);
				});

				// if there's an error with the InAppBrowser loading the url
				win.addEventListener( 'loaderror', function() {
					win.close();
			  		nativeAlert(
			  			'Please check your internet connection and try again.',
			  			function() {
			  				// disable Dropbox Save & Restore option
					  		$dropboxSaveAndRestore.removeAttr('checked');
			  			},
			  			'Could not connect to Dropbox!'
			  		);
				});

				// InAppBrowser is closed by user
				win.addEventListener( 'exit', function() {
					if (!localStorage.access_token) {
				  		nativeAlert(
				  			'Could not link to Dropbox because the popup was closed. Please try enabling Dropbox again.',
				  			function() {
				  				// disable Dropbox Save & Restore option
						  		$dropboxSaveAndRestore.removeAttr('checked');
				  			},
				  			'Notice'
				  		);
					}
				});

			}

			// confirm dropbox active
			function confirmDropboxActive() {

				nativeAlert(
					'You can now save your data to Dropbox at any time by tapping "Save data to Dropbox" here on the Settings page, or from the "Options" menu when viewing your budgets.',
		    		function() {
		    			// check for existing data
				  		balance.webdb.checkForData();
		    		},
					'Dropbox Save & Restore active!'
				);

			}

		/*
			>> check data on dropbox
		*/

			/*$('#setting-check-database').click(function (e) {
				balance.webdb.checkForData();
			});*/

		/*
			>> dropbox save database
		*/

			$('.save-data-to-dropbox').click(function (e) {
				e.preventDefault();
				console.log('Clicked Save data to Dropbox');
				balance.webdb.saveDatabase();
			});

		/*
			>> dropbox restore database
		*/

			var restoreClickEvents = (platform == 'Browser' ? ['mousedown', 'mouseup'] : ['touchstart', 'touchend'])

			// restore previous revision (click and hold)
			$('.restore-data-from-dropbox').on(restoreClickEvents[0], function() {
			    restorePreviousRevision = setTimeout(function() {
			    	gettingRevisions = true;
			    	balance.webdb.getRevisions();
			    }, 2000);
			}).bind(restoreClickEvents[1], function() {
			    clearTimeout(restorePreviousRevision);
			});

			// normal restore
			$('.restore-data-from-dropbox').click(function (e) {
				e.preventDefault();
				if (!gettingRevisions) {
					console.log('Clicked Restore data from Dropbox');
					balance.webdb.restoreDatabase(balance.webdb.getAllBudgets);
				}
			});

			// choose revision to restore
			$('#choose-revision').change(function (e) {
				var $this = $(this);
				var label = $('#choose-revision').children(':selected').text();
				var rev = $this.val();
				if ($this.val() !== '') {

					// restore revision confirm
					nativeConfirm(
						'Restore revision "' + label + '"?',	// message
						restoreRevisionCallback,				// callback
						'Restore revision?',					// title
						['No', 'Yes'],							// button labels
						null,									// fallback
						2,										// fallback - ok pressed
						1										// fallback - cancel pressed
					);

					// restore revision confirm callback
					function restoreRevisionCallback(buttonIndex) {
						if (buttonIndex === 2) {
							balance.webdb.restoreRevision(rev);
						}
						else {
							$('#choose-revision').val('');
						}
					}
				}
			});

		/*
			>> persistent storage - restore
		*/

			$('.restore-data-from-persistent-storage').click(function (e) {

				persistent.restoreDatabase(balance.webdb.getAllBudgets);

			});


		/*
			>> persistent storage - stop asking
		*/

			$('.my-data-is-fine').click(function (e) {

				persistent.stopAsking();

			});


		/*
			>> persistent storage - save
		*/

			$('.save-data-to-persistent-storage').click(function (e) {

				persistent.saveDatabase();

			});


		/*
			>> persistent storage - check if database deleted by ios
		*/

			$('.check-if-database-deleted-by-ios').click(function (e) {

				persistent.checkIfDatabaseDeletedByIOS(balance.webdb.getAllBudgets);

			});

	}


/* -----------------------------------------------------------------------------------
	> onAppPaused
--------------------------------------------------------------------------------------- */

	function onAppPaused() {
		// if there's a pin code, log user out and prompt for pin code when they come back
		if (localStorage.pinCode !== undefined && localStorage.pinCode.length > 0) {
			doLogin();
		}
	}


/* -----------------------------------------------------------------------------------
	> onAppResumed
--------------------------------------------------------------------------------------- */

	function onAppResumed() {
		
	}
