/*
* CONTENTS
*
* setup
* save database
*   backup all tables
*   serialize data
*   save to persistent storage
*   fn: backup a table to json
*   fn: save serialized db to persistent storage
* restore database
* check if database deleted by ios
*   get number of budgets in persistent storage database backup
* stop asking
* fn: do restore
*   get link to db
*   delete all of the table rows
*   restore all of the tables
*/

	/*
		> setup
	*/

		var persistent = {};
		var fileName = 'Fudget.data';

		// enable emergency mode on load if we're in emergency mode
		if (localStorage.emergencyMode) {
			enableEmergencyMode();
		}

	/*
		> save database
	*/

		var data = {};

		persistent.saveDatabase = function() {

			if (isNative && !localStorage.emergencyMode) {

				data = {};

				// >> backup all tables
				$.when(
				    backup('budgets'), 
				    backup('entries'),
				    backup('budgets_entries'),
				    backup('paid')
				).then(function(jsonData) {
					// get number of budgets from web sql db
					var numberOfBudgets = jsonData.budgets.length;

					// log('* numberOfBudgets: ', numberOfBudgets);

					// if there's at least 1 budget, save to persistent storage
					if (numberOfBudgets > 0) {
					    // >> serialize data
					    var serializedData = JSON.stringify(jsonData);
					    // >> save to persistent storage
					    saveSerializedDbToPersistentStorage(serializedData);
					}

				});

			}

		}

		// >> fn: backup a table to json
		function backup(table) {
			var db = balance.webdb.db;
		    var def = new $.Deferred();
		    data[table] = [];
		    db.transaction(function(tx) {
		    	// select everything from table
		        tx.executeSql("select * from "+table, [], function(tx,results) {
		        	for (var i=0; i < results.rows.length; i++) {
		        		// loop through each key in the row and push only the value to an array
		        		var rowArray = [];
		        		for (var key in results.rows.item(i)) {
		        		  rowArray.push(results.rows.item(i)[key]);
		        		}
		        		// push the array of only values to the data object
		        		data[table].push(rowArray);
		        	}
		            // var data = results;
		            def.resolve(data);
		        });
		    }, persistent.onError);

		    return def;
		}

		// >> fn: save serialized db to persistent storage
		function saveSerializedDbToPersistentStorage(serializedData) {

	    	// log('saveSerializedDbToPersistentStorage()');

    		// request persistent storage
    		window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fs) {
    			// log('file system open: ' + fs.name);

    			// Creates a new file or returns the file if it already exists.
    		    fs.root.getFile(fileName, {create: true, exclusive: false}, function(fileEntry) {
    		        writeFile(fileEntry, serializedData);
    		    }, function() {
    		    	// log('Error creating file');
    		    });

    		}, function(error) {
    			alert('error - file system not loaded');
    		});

    	    // fn: write file
    	    function writeFile(fileEntry, dataObj) {

    	        // Create a FileWriter object for our FileEntry (log.txt).
    	        fileEntry.createWriter(function (fileWriter) {

    	            fileWriter.onwriteend = function() {
    	                // log("Successful file write...");
    	            };

    	            fileWriter.onerror = function (e) {
    	                // log("Failed file write: " + e.toString());
    	            };

    	            fileWriter.write(dataObj);

    	        });
    	    }

		}


	/*
		> restore database 
	*/

		persistent.restoreDatabase = function(callback) {

			if (isNative) {

				// request persistent storage
				window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fs) {
					// log('file system open: ' + fs.name);

					// Creates a new file or returns the file if it already exists.
				    fs.root.getFile(fileName, {create: true, exclusive: false}, function(fileEntry) {
				        readFile(fileEntry);
				    }, function() {
				    	// error
				    });

				}, function(error) {
					alert('error - file system not loaded');
				});				

			    // fn: read file
			    function readFile(fileEntry) {
			        fileEntry.file(function (file) {
			            var reader = new FileReader();

			            // succesfully loaded file
			            reader.onloadend = function() {
			                // log("Successful file read: " + this.result);

			                var result = this.result;

			                nativeConfirm(
			                	'Attempt to restore your lost data?',	    // message
			                	restoreLostDataConfirmCallback,	            // callback (with tapped button index as parameter)
			                	'Restore your lost data',		            // title
			                	['Yes', 'Not now', 'Stop asking'],			// button labels
			                	'Not supported in browser'				    // fallback (null if confirm fallback needed, string or function)
			                );

			                function restoreLostDataConfirmCallback(buttonIndex) {
			                	if (buttonIndex === 1) {
					                // do the restore
					                doRestore(result, callback);
			                	}
			                	else if (buttonIndex === 3) {
			                		// prompt to stop asking
			                		persistent.stopAsking();
			                	}
			                }

			            };

			            reader.readAsText(file);

			        }, function() {

			        });
			    }

			}

		}


	/*
		> check if database deleted by ios
	*/

		persistent.checkIfDatabaseDeletedByIOS = function(callback) {

			if (isNative) {

				// get number of budgets in database
				var db = balance.webdb.db;
				db.transaction(function(tx) {
					tx.executeSql('SELECT * FROM budgets', [], function(tx,rs) {

						// number of budgets in database
						var numberOfDbBudgets = rs.rows.length;

						/*
							>> get number of budgets in persistent storage database backup
						*/

							// request persistent storage
							window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fs) {
								// log('file system open: ' + fs.name);

								// Creates a new file or returns the file if it already exists.
							    fs.root.getFile(fileName, {create: true, exclusive: false}, function(fileEntry) {
							        readFile(fileEntry);
							    }, function() {
							    	// error
							    });

							}, function(error) {
								alert('error - file system not loaded');
							});				

						    // fn: read file
						    function readFile(fileEntry) {
						        fileEntry.file(function (file) {
						            var reader = new FileReader();

						            // succesfully loaded file
						            reader.onloadend = function() {
						            	var stringResult = this.result; 
						            	var jsonResult = JSON.parse(stringResult);

						                // number of budgets in persistent storage db
						            	var numberOfPersistentStorageBudgets = jsonResult.budgets.length; 

						            	// log('numberOfDbBudgets: ', numberOfDbBudgets);
						                // log("numberOfPersistentStorageBudgets: " + numberOfPersistentStorageBudgets);

						                // if there are more budgets in persistent storage than in the db,
						                // warn user and offer to restore
						                if (numberOfDbBudgets === 0 && numberOfPersistentStorageBudgets > numberOfDbBudgets) {

						                	// enable emergency mode
						                	enableEmergencyMode();

						                	nativeAlert(
						                		'Your data may have been deleted by iOS. If you have lost your data, please go to Settings and tap "Restore my lost data".',		// message
						                		function() {	// callback
						                		},		
						                		'Possible data loss!',		// title
						                		'OK'			// button label
						                	);

						                }

						            };

						            reader.readAsText(file);

						        }, function() {

						        });
						    }


					}, null);
				});	

			}

		}


	/*
		> stop asking
	*/

		persistent.stopAsking = function() {

			nativeConfirm(
				"Are you sure you want me to stop asking? You won't get another chance to restore lost data.",	// message
				stopAskingCallback,					// callback (with tapped button index as parameter)
				'Really stop asking?',						// title
				['Yes, stop asking', 'No'],							// button labels
				null,									// fallback (null if confirm fallback needed, string or function)
				1,										// fallback - ok pressed (button index to fire - omit if no confirm required)
				2										// fallback - cancel pressed (button index to fire)
			);

			function stopAskingCallback(buttonIndex) {
				if (buttonIndex === 1) {
					disableEmergencyMode();
				}
			}


		}


	/*
		> fn: do restore 
	*/

    	function doRestore(data, callback) {

            var jsonString = data;
            var jsonData = JSON.parse(jsonString);

            // log(jsonData);

   			// >> get link to db
   			var db = balance.webdb.db;

   		    // >> delete all of the table rows
   		    $.when(
   		        deleteTable('budgets', db), 
   		        deleteTable('entries', db),
   		        deleteTable('budgets_entries', db),
   		        deleteTable('paid', db)
   		    ).then(function() {
   		    	// >> restore all of the tables
   		    	$.when(
   		    		restoreTable('budgets', db, jsonData),
   		    		restoreTable('entries', db, jsonData),
   		    		restoreTable('budgets_entries', db, jsonData),
   		    		restoreTable('paid', db, jsonData)
   		    	).then(function(){

   		    		// log('Database restored!');

   		    		// show notification
   		    		showNotification('Data restored from persistent storage!', 'success', 3000);

   		    		// disable emergency mode
   		    		disableEmergencyMode();

   		    		// fire the callback if there is one
   		    		if (callback) {
   		    			callback();
   		    		}
   		    	});

   		    });
    	}