/*
* database-sync.js
*/

/*
* CONTENTS
*
* check database
* save database
*   backup all tables
*   serialize data
*   save to dropbox file
*   fn: backup a table to json
*   fn: save serialized db to dropbox
* restore database
*   open file from dropbox
*   get link to db
*   delete all of the table rows
*   restore all of the tables
*   fn: delete table
*   fn: restore table
* generic functions
*   fn: generate sync id
*   fn: start checking for changes on dropbox
*    fn: start polling for changes
*    fn: function to update all the views after data has changed
*   fn: start checking for changes locally
*   fn: start checking for changes
*/
		
	
	/*
		> check database
		  see if there is already data on Dropbox first time user logs in
	*/

		balance.webdb.checkDatabase = function(callback) {

			// arguments
            var args = JSON.stringify({
            	'path': '/Fudget.data'
			});

            $.ajax({
                url: 'https://api.dropboxapi.com/2/files/get_metadata',
                type: 'POST',
                beforeSend: function (xhr) {
                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
                    xhr.setRequestHeader('Content-Type', 'application/json');
                },
                data: args,
                // data found on dropbox
                success: function( response ) {
                	console.log('Data found on Dropbox');
		       		if (isNative) {
		       			navigator.notification.confirm(
			       		    'Download the Dropbox data to this device? \n\r Note: this will replace the data already on this device', // message
			       		     function(index) {
			       		     	// yes pressed
			       		     	if (index === 1) {
			       		     		yesDownloadDropboxData();	
			       		     	}
			       		     	// no pressed
			       		     	else if (index === 2) {
			       		     		noUseLocalData();
			       		     	}
			       		     },
			       		    'Data found on Dropbox', // title
			       		    ['Yes, download Dropbox data','No, use data from this device']);

		       		}
		       		else {
		       			var confirmDownloadDropboxData = confirm('Download the Dropbox data to this device? \n\r Note: this will replace the data already on this device');
						if (confirmDownloadDropboxData === true) {
							yesDownloadDropboxData();
						}
						else {
							noUseLocalData();
						}
		       		}
		       		function yesDownloadDropboxData() {
		       			balance.webdb.restoreDatabase(updateViews);
		       		}
		       		function noUseLocalData() {
		       			balance.webdb.saveDatabase(startCheckingForChangesOnDropbox);
		       		}
	       			
                },
                // no data found on dropbox
				error: function(XMLHttpRequest, textStatus, errorThrown) {
					var responseText = XMLHttpRequest.responseText;
					// if no file found, save the user's current data to dropbox and start checking for changes
					if (responseText.indexOf('not_found') > -1) {
						console.log('No data on dropbox');
						startCheckingForChanges();
					}
				}
            });

		}


	/*
		> save database
	*/
	
		var data = {};

		balance.webdb.saveDatabase = function(callback) {

			// only if auto sync enabled
			if (localStorage.autoSync === 'true') {

				data = {};
				// >> backup all tables
				$.when(
				    backup('budgets'), 
				    backup('entries'),
				    backup('budgets_entries'),
				    backup('paid')
				).then(function(jsonData) {
					// console.log(jsonData);
					// add a random sync id to the json
					var newSyncId = generateSyncId();
					jsonData['syncId'] = newSyncId;
					// add the sync date to the json (milliseconds since epoch)
					var newSyncDate = new Date().getTime();
					jsonData['syncDate'] = newSyncDate;
				    // >> serialize data
				    var serializedData = JSON.stringify(jsonData);
				    // >> save to dropbox file
				    saveSerializedDbToDropbox(serializedData, newSyncId, newSyncDate, callback);
				});

			}

		}

		// >> fn: backup a table to json
		function backup(table) {
			var db = balance.webdb.db;
		    var def = new $.Deferred();
		    data[table] = [];
		    db.transaction(function(tx) {
		    	// select everything from table
		        tx.executeSql("select * from "+table, [], function(tx,results) {
		        	for (var i=0; i < results.rows.length; i++) {
		        		// loop through each key in the row and push only the value to an array
		        		var rowArray = [];
		        		for (var key in results.rows.item(i)) {
		        		  rowArray.push(results.rows.item(i)[key]);
		        		}
		        		// push the array of only values to the data object
		        		data[table].push(rowArray);
		        	}
		            // var data = results;
		            def.resolve(data);
		        });
		    }, balance.webdb.onError);

		    return def;
		}

		// >> fn: save serialized db to dropbox
		function saveSerializedDbToDropbox(serializedData, newSyncId, newSyncDate, callback) {

			// if zipping is enabled, zip the data first
			if (dropboxZipMode) {
				console.log('dropboxZipMode enabled');
				// zip the serialized data
				var compression_mode = 9,
				    zip = LZMA; /// lzma_worker.js creates a global LZMA object.

			    /// First, let's compress it.
			    zip.compress(serializedData, compression_mode, function on_compress_complete(result) {

			    	var data = JSON.stringify(result);
			    	console.log(data);
			    	uploadToDropbox(data);

			    }, null);
			}
			// else don't zip the data
			else {
				uploadToDropbox(serializedData);
			}

		    // fn: upload to dropbox
		    function uploadToDropbox(result) {
		    	console.log('result', result);
				// arguments
	            var args = JSON.stringify({
	            	'path': '/Fudget.data',
				    'mode': 'overwrite', // add, overwrite, update
				    'autorename': false,
				    'mute': false
				});
				// upload to dropbox
	            $.ajax({
	                url: 'https://content.dropboxapi.com/2/files/upload',
	                type: 'POST',
	                beforeSend: function (xhr) {
	                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
	                    xhr.setRequestHeader('Content-Type', 'application/octet-stream');
	                    xhr.setRequestHeader('Dropbox-API-Arg', args);
	                },
	                data: result,
	                success: function( response ) {
	                    // alert('response' + response);
	                    log('Database saved to Dropbox!');
	                    console.log('Database saved to Dropbox!');
	                    // save the syncId to localStorage
	                    localStorage.syncId = newSyncId;
	                    // save the syncDate to localStorage
	                    localStorage.syncDate = newSyncDate;
	                    // we no longer have any unpushed local changes
	                    if (unpushedLocalChanges) {
	                    	showNotification('Unpushed local changes saved to Dropbox', 'success');
		                    unpushedLocalChanges = false;
	                    }
	                    // start checking for changes locallly
	                    startCheckingForChangesLocally();
	                    if (callback) {
	                    	callback();
	                    }

	                },
					error: function(XMLHttpRequest, textStatus, errorThrown) {
						console.dir(XMLHttpRequest);
						console.log(XMLHttpRequest.responseText);
						// alert(textStatus + ' ~ ' + errorThrown);
						showNotification('Could not save data to Dropbox: ' + textStatus, 'error');
						unpushedLocalChanges = true;
						startCheckingForChangesLocally();
					}
	            });				    	
		    }

		}


	/*
		> restore database
	*/

		balance.webdb.restoreDatabase = function(callback) {

			// only if auto sync enabled
			if (localStorage.autoSync === 'true') {

				console.log('Start restoring database');

				// arguments
	            var args = JSON.stringify({
	            	'path': '/Fudget.data'
				});
				// >> open file from dropbox
	            $.ajax({
	                url: 'https://content.dropboxapi.com/2/files/download',
	                type: 'POST',
	                beforeSend: function (xhr) {
	                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
	                    xhr.setRequestHeader('Dropbox-API-Arg', args);
	                },
	                success: function( response ) {

	                	// if user previously tapped try again on coulndtConnect notification, show we successfully connected
	                	if (userTappedTryAgainOnCouldntConnect) {
	                		showNotification('Successfully connected to Dropbox!', 'success', 4000);
	                		userTappedTryAgainOnCouldntConnect = false;
	                	}

	                	// if zipping is enabled, unzip the data first
	                	if (dropboxZipMode) {

		                	var zipArray = JSON.parse(response);

		                	/// unzip the zipped code
		                	zip = LZMA; /// lzma_worker.js creates a global LZMA object.
		                	zip.decompress(zipArray, function on_decompress_complete(result) {

		                		doRestore(result);

		                	}, null);

	                	}
	                	// else don't unzip the data
	                	else {
	                		doRestore(response);
	                	}

	                	// fn: doRestore
	                	function doRestore(result) {

		                    var jsonString = result;
		                    var jsonData = JSON.parse(jsonString);

		                    // only if data on the server is newer and has different sync id
		                    // OR if user has never restored before
		                    var dropboxSyncDate = parseInt(jsonData['syncDate']),
		                    	localSyncDate = parseInt(localStorage.syncDate),
		                    	dropboxSyncId = jsonData['syncId'],
		                    	localSyncId = localStorage.syncId;
		                    	// console.log(dropboxSyncDate, localSyncDate, dropboxSyncId, localSyncId);

		                    // if new data, restore
		                   	// if (dropboxSyncDate > localSyncDate && dropboxSyncId !== localSyncId || !localStorage.previouslyRestored) {
		                   	// NOTE: stopped checking sync date as the app failed to sync if one device had the wrong device time set
		                   	if (dropboxSyncId !== localSyncId || !localStorage.previouslyRestored) {
		                   		// log('dropbox data is newer and sync id is different OR user has not restored before, so do the restore');
		                   		console.log('new data');

		                   		// show loading screen if someone else updated data and we started syncing
		                   		if (callback.name === 'updateViews') { 
			                   		showLoadingScreen('Data changed elsewhere. Syncing...');
			                   	}

	                   			// >> get link to db
	                   			var db = balance.webdb.db;

	                   		    // >> delete all of the table rows
	                   		    $.when(
	                   		        deleteTable('budgets', db), 
	                   		        deleteTable('entries', db),
	                   		        deleteTable('budgets_entries', db),
	                   		        deleteTable('paid', db)
	                   		    ).then(function() {
	                   		    	// >> restore all of the tables
	                   		    	$.when(
	                   		    		restoreTable('budgets', db, jsonData),
	                   		    		restoreTable('entries', db, jsonData),
	                   		    		restoreTable('budgets_entries', db, jsonData),
	                   		    		restoreTable('paid', db, jsonData)
	                   		    	).then(function(){
	                   		    		// restore syncId
	                   		    		localStorage.syncId = jsonData['syncId'];
	                   		    		// restore syncDate
	                   		    		localStorage.syncDate = jsonData['syncDate'];
	                   		    		// flag to show if user has restored a database 
	                   		    		// at least once
	                   		    		localStorage.previouslyRestored = true;
	                   		    		log('Database restored!');
	                   		    		console.log('Database restored!');
	                   		    		// fire the callback if there is one
	                   		    		if (callback) {
	                   		    			callback();
	                   		    		}
	                   		    	});

	                   		    });
		                   	}
		                   	// else fire callback (unless it's the updateViews function)
		                   	else if (callback && callback.name !== 'updateViews') {
		                   		callback();
		                   	}
		                   	// if it's the updateViews function, just start checking for changes on dropbox again
		                   	else if (callback.name === 'updateViews') {
		                   		startCheckingForChangesOnDropbox();
		                   		// if not checking for local changes, start checking for local changes
	                   			startCheckingForChangesLocally();
		                   	}

	                	} 


	                },
					error: function(XMLHttpRequest, textStatus, errorThrown) {
						console.dir(XMLHttpRequest);
						console.log(XMLHttpRequest.responseText);
						// log('Could not connect to Dropbox.');
						// alert(textStatus + ' ~ ' + errorThrown);
						// showNotification('Could not connect to Dropbox: ' + textStatus, 'error');
						// notify user we couldn't connect to Dropbox
			       		if (isNative) {

			       			// if user has not previously dismissed the could not connect notifictation (below)
			       			if (!dismissedCouldntConnect) {
				       			navigator.notification.confirm(
					       		    'Could not connect to Dropbox. Please check your internet and try again.', // message
					       		     function(index) {
					       		     	// Try again pressed
					       		     	if (index === 1) {
					       		     		couldntConnectTryAgain();	
					       		     	}
					       		     	// Continue pressed
					       		     	else if (index === 2) {
					       		     		couldntConnectContinue();
					       		     	}
					       		     },
					       		    'Couldn\'t connect to Dropbox', // title
					       		    ['Try again','Continue & sync later']
					       		);

					       		function couldntConnectTryAgain() {
					       			balance.webdb.restoreDatabase(balance.webdb.getAllBudgets);
					       			userTappedTryAgainOnCouldntConnect = true;
					       		}
					       		function couldntConnectContinue() {
					       			confirmBeforeRestore = true;
					       			dismissedCouldntConnect = true;
					       		}
			       			}
			       			// else just show a little notification rather than the full confirm box again
			       			else {
			       				showNotification('Could not connect to Dropbox: ' + textStatus, 'error');
			       			}

			       		}
						if (callback) {
							callback();
						}
					}
	            });		

			}

			// else go straight to the callback (if there is on)
			else if (callback) {
				callback();
			}

		}

		// >> fn: delete table
		function deleteTable(table, db) {
			var def = new $.Deferred();
		    db.transaction(function(tx) {
		        tx.executeSql("DELETE FROM " + table, [], function(tx,results) {
		            def.resolve(results);
		        });
		    }, function() {console.log('delete table error');});

		    return def;	
		}

		// >> fn: restore table
		function restoreTable(table, db, jsonData) {
			db.transaction(function(tx) {
        		// build up the question marks part of the query based on the number of columns e.g. ?, ?, ?
        		q = '';
        		for (var j in jsonData[table][0]) {
        			q += (q == "" ? "" : ", ") + "?";
        		}
        		
        		// for each row of this table
        		for (var i=0; i < jsonData[table].length; i++) {
        			var query = 'INSERT OR IGNORE INTO ' + table + ' VALUES (' + q + ')';
					tx.executeSql(query, jsonData[table][i], null, balance.webdb.onError);
				}

			});
		}


	/*
		> generic functions
	*/

		// >> fn: generate sync id
		function generateSyncId() {
		    var text = "";
		    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

		    for( var i=0; i < 15; i++ )
		        text += possible.charAt(Math.floor(Math.random() * possible.length));

		    return text;
		}

		// >> fn: start checking for changes on dropbox
		var checkForChangesOnDropbox; // a variable for the interval
		var cursor = '';

		function startCheckingForChangesOnDropbox() {
			// only if auto sync enabled
			if (localStorage.autoSync === 'true') {
				log('Start checking for changes on Dropbox');
				console.log('Start checking for changes on Dropbox');

				// if not currently checking for changes on dropbox
				if (!currentlyCheckingForChangesOnDropbox) {

					currentlyCheckingForChangesOnDropbox = true;

					// if we don't have a cursor, get a cursor
					if (cursor === '') {
						// arguments
			            var args = JSON.stringify({
			            	'path': ''
						});
			            $.ajax({
			                url: 'https://api.dropboxapi.com/2/files/list_folder/get_latest_cursor',
			                type: 'POST',
			                beforeSend: function (xhr) {
			                    xhr.setRequestHeader('Authorization', 'Bearer ' + localStorage.access_token);
			                    xhr.setRequestHeader('Content-Type', 'application/json');
			                },
			                data: args,
			                success: function( response ) {
			                	// console.log(response);
			                	cursor = response.cursor;
			                	startPolling(cursor);

			                },
							error: function(XMLHttpRequest, textStatus, errorThrown) {
								console.dir(XMLHttpRequest);
								console.log(XMLHttpRequest.responseText);
								currentlyCheckingForChangesOnDropbox = false;
								// try again in x minutes
								setTimeout(function(){
									startCheckingForChangesOnDropbox();
								}, tryDropboxAgainTimeout);
							}
			            });		
					}
					// else start polling
					else {
						startPolling(cursor);
					}

				}


			}
		}

			// >>> fn: start polling for changes
			function startPolling(thisCursor) {
				// arguments
	            var args = JSON.stringify({
	            	'cursor': thisCursor
				});
	            $.ajax({
	                url: 'https://notify.dropboxapi.com/2/files/list_folder/longpoll',
	                type: 'POST',
	                beforeSend: function (xhr) {
	                    xhr.setRequestHeader('Content-Type', 'application/json');
	                    xhr.setRequestHeader('Access-Control-Allow-Origin', '*');
	                },
	                data: args,
	                success: function( response ) {
	                	console.log(response);
	                	if (localStorage.getItem('autoSync') === 'true') {

	                		// let user know we checked for changes ok, if they previously coulndn't check for changes
	                		if (sawCouldntCheckForChangesOnDropbox) {
	                			showNotification('Successfully checked for changes on Dropbox!', 'success');
	                			sawCouldntCheckForChangesOnDropbox = false;
	                		}

		                	// if changes were made, check if we should restore the database
		                	if (response.changes === true) {
		                		console.log('Changes were made on Dropbox');
		                		cursor = '';
		                		// if user was previously unable to connect to dropbox, and dismissed the 'couln't connect to dropbox' confirmation, then get their confirmation before restoring
		                		currentlyCheckingForChangesOnDropbox = false;
		                		if (dismissedCouldntConnect) {
        			       			navigator.notification.confirm(
        				       		    'New data found on Dropbox. Download and overwrite your data?', // message
        				       		     function(index) {
        				       		     	// Download pressed
        				       		     	if (index === 1) {
						                		balance.webdb.restoreDatabase(updateViews);
        				       		     	}
        				       		     	// No, use my data pressed
        				       		     	else if (index === 2) {
        				       		     		balance.webdb.saveDatabase(startCheckingForChangesOnDropbox);
        				       		     	}
        				       		     	dismissedCouldntConnect = false;
        				       		     },
        				       		    'New data found on Dropbox', // title
        				       		    ['Download ','No, use my data']
        				       		);
		                		}
		                		// else just do the restore 
		                		else {
			                		balance.webdb.restoreDatabase(updateViews);
			                	}
		                	}
		                	// otherwise, poll again
		                	else {
		                		startPolling(cursor);
		                	}

		                }
		                else {
		                	console.log('stop checking for changes on dropbox');
		                	currentlyCheckingForChangesOnDropbox = false;
		                }
	                },
					error: function(XMLHttpRequest, textStatus, errorThrown) {
						console.dir(XMLHttpRequest);
						console.log(XMLHttpRequest.responseText);
						console.log('longpoll error');
						currentlyCheckingForChangesOnDropbox = false;
						if (!sawCouldntCheckForChangesOnDropbox) {
							showNotification('Couldn\'t check for changes on Dropbox', 'error');
							sawCouldntCheckForChangesOnDropbox = true;
						}
						// try again in x minutes
						setTimeout(function(){
							startCheckingForChangesOnDropbox();
						}, tryDropboxAgainTimeout);
					}
	            });
			}

			// >>> fn: function to update all the views after data has changed
			function updateViews() {
				console.log('updateViews');
				balance.webdb.getAllBudgets();
			}

		// >> fn: start checking for changes locally
		var localChanges = false;
		var checkingForChangesLocally = false;
		var localChangesInterval;
		function startCheckingForChangesLocally() {
			// only if not already checking for changes locally
			if (!checkingForChangesLocally) {
				// only of it syncing is enabled
				if (localStorage.getItem('autoSync') === 'true') {
					log('Start checking for changes locally.');
					checkingForChangesLocally = true;
					console.log('Start checking for changes locally.');
					localChangesInterval = setInterval(function(){
						if (localStorage.getItem('autoSync') === 'true') {
							console.log('see if there are changes locally');
							if (localChanges === true || unpushedLocalChanges === true) {
								console.log('Local changes found. Stop checking for local changes.');
								localChanges = false; 
								// clear interval
								clearInterval(localChangesInterval);
								checkingForChangesLocally = false;
								// save database
								balance.webdb.saveDatabase();
							}
						}
						else {
							console.log('stop checking for changes locally');
							clearInterval(localChangesInterval);
							console.log('clear interval');
							checkingForChangesLocally = false;
						}
					}, 5000);
				}
			}
		}

		// >> fn: start checking for changes
		function startCheckingForChanges() {
			startCheckingForChangesLocally();
			startCheckingForChangesOnDropbox();
		}