/*
* database.js
*
* CONTENTS
*
* open the database
* delete the database
* create tables
* check for errors
* database opened successfully
* get all budgets
* add budget
* edit budget
* update budget order number
* delete budget
* get all entries
* add entry
* edit entry
* update entry order number
* delete entry
* pin entry
* unpin entry
*/

	var balance = {};
	balance.webdb = {};
	balance.webdb.db = null;
	
	/*
		> open the database
	*/
		
		balance.webdb.open = function() {
			var dbSize = 1024 * 1024 * 5; // 5MB
			balance.webdb.db = openDatabase('Balance', '1.0', 'Balance iPhone App', dbSize);
		}
		

	/*
		> delete the database
	*/

		balance.webdb.deleteDatabase = function() {
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				tx.executeSql("DROP TABLE budgets", [], null, balance.webdb.onError);
				tx.executeSql("DROP TABLE entries", [], null, balance.webdb.onError);
				tx.executeSql("DROP TABLE budgets_entries", [], null, balance.webdb.onError);
			});
		}


	/*
		> create tables
	*/
	
		balance.webdb.createTables = function() {
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				// budgets
				tx.executeSql('CREATE TABLE IF NOT EXISTS budgets(id INTEGER PRIMARY KEY ASC AUTOINCREMENT, title TEXT, order_number INTEGER, date_created DATETIME)', [], null, balance.webdb.onError);
				// entries
				tx.executeSql('CREATE TABLE IF NOT EXISTS entries(id INTEGER PRIMARY KEY ASC AUTOINCREMENT, title TEXT, amount FLOAT, pinned INTEGER DEFAULT 0, isexpense INTEGER DEFAULT 0, date_created DATETIME)', [], null, balance.webdb.onError);
				// budgets_entries
				tx.executeSql('CREATE TABLE IF NOT EXISTS budgets_entries(id INTEGER PRIMARY KEY ASC AUTOINCREMENT, budget_id INTEGER, entry_id INTEGER, order_number INTEGER)', [], null, balance.webdb.onError);
				// paid
				tx.executeSql('CREATE TABLE IF NOT EXISTS paid(id INTEGER PRIMARY KEY ASC AUTOINCREMENT, budgets_entries_id INTEGER)', [], null, balance.webdb.onError);

			});	
		}
		
	/*
		> check for errors
	*/ 
	
		balance.webdb.onError = function(tx, error){
			// alert('There has been an error: ' + error.message);	
			console.log('There has been an error:');
			console.dir(error);	
			// navigator.notification.alert('There has been an error: ' + error.message);	
		}
	
		
	/*
		> database opened successfully
	*/
		
		balance.webdb.onSuccess = function(tx, r) {
			
		}
		
		
	/*
		> get all budgets
	*/
	
		balance.webdb.getAllBudgets = function() {
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				tx.executeSql('SELECT * FROM budgets ORDER BY order_number', [], loadAllBudgets, balance.webdb.onError);
			});	
		}
		
	/*
		> add budget
	*/
	
		balance.webdb.addBudget = function(orderNumber, $budgetMarkup) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				var title = '';
				var dateCreated = new Date();
				
				// get id of previous budget from 'budgets' 
				var previousBudgetId;
				tx.executeSql('SELECT id FROM budgets ORDER BY id DESC LIMIT 1', [],
					function(tx, results) {
						try {
							previousBudgetId = results.rows.item(0).id;
						}
						catch (e) {
							previousBudgetId = 0;	
						}

						// insert new budget into 'budgets'
						tx.executeSql('INSERT INTO budgets(title, order_number, date_created) VALUES (?,?,?)', [title, orderNumber, dateCreated], 
							function(tx, results) {
								// get id of new budget
								newBudgetId = results.insertId;
								
								// add the budget markup
								addIdToBudgetMarkup($budgetMarkup, newBudgetId);

								localChanges = true;
							},
							balance.webdb.onError);

					},
					balance.webdb.onError);
				
			});
		}
		
	/*
		> edit budget
	*/
	
		balance.webdb.editBudget = function(id, title) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {				
				tx.executeSql("UPDATE budgets SET title = ? WHERE id = ?", [title, id], function(tx, rs) {
					localChanges = true;

				}, balance.webdb.onError);
			});
		}
		
		
	/*
		> update budget order numbers
	*/		
	
		balance.webdb.updateBudgetOrderNumbers = function(budgetIds, orderNumbers) {
			var noRows = budgetIds.length;
			var doneRows = 0;
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				for (i=0; i<budgetIds.length; i++) {
					tx.executeSql("UPDATE budgets SET order_number = ? WHERE id = ?", [orderNumbers[i], budgetIds[i]], function(tx, rs) {
						doneRows++;
					}, balance.webdb.onError);
				}
			});
			// wait til all rows have been updated before saving database to dropbox
			var checkDone = setInterval(function(){
				if (doneRows === noRows) {
					clearInterval(checkDone);
					localChanges = true;
				}
			}, 100);
		}
		
		
	/*
		> delete budget
	*/
	
		balance.webdb.deleteBudget = function(id) {
			var db = balance.webdb.db;
			db.transaction(function(tx){
				// budgets_entries - create array of entry_id's where budget_id = id (e.g. 4, 5, 6)
				entryIds = new Array;
				tx.executeSql('SELECT entry_id FROM budgets_entries WHERE budget_id = ?', [id], 
					function(tx, rs) {
						// create array of entry id's (but not first one)
						for (i=0; i < rs.rows.length; i++) {
							entryIds.push(rs.rows.item(i).entry_id);
						}
						var noEntryIds = entryIds.length;
						var noEntriesDeleted = 0;
						// entries - using array, delete where id like array value and pinned != 1
						for (j = 0; j < entryIds.length; j++) {
							currentEntryId = entryIds[j];
							tx.executeSql('DELETE FROM entries WHERE id=? AND pinned=0', [currentEntryId], function(tx, rs) {
								noEntriesDeleted++;
							}, balance.webdb.onError);
						}

						// wait til all entries have been deleted before deleting from budgets_entries and budgets
						var checkEntriesDeletedDone = setInterval(function(){
							if (noEntriesDeleted === noEntryIds) {
								clearInterval(checkEntriesDeletedDone);
								db.transaction(function(tx) {
									// budgets_entries - delete * from budgets_entries where budget_id = id
									tx.executeSql('DELETE FROM budgets_entries WHERE budget_id=?', [id], function(tx, rs) {
										// budgets - delete from budgets where id = id
										tx.executeSql('DELETE FROM budgets WHERE ID=?', [id], function(tx, rs) {
											localChanges = true;
										}, balance.webdb.onError);
									}, balance.webdb.onError);
								});
							}
						}, 100);
						
				}, balance.webdb.onError);
				
			});	
		}
	
	/*
		> add pinned entries to budget
	*/

		balance.webdb.addPinnedEntriesToBudget = function(budgetId) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				// get all ids of pinned entries from 'entries' into an array - e.g. 1, 2
				var pinnedEntryIds = new Array;
				tx.executeSql('SELECT id FROM entries WHERE pinned = ?', [1], 
					function(tx, results) {
						for (var i=0; i < results.rows.length; i++) {
							pinnedEntryIds.push(results.rows.item(i).id);
						}
						// get order_number of each pinned entry based on previousBudgetId and 'entry_id' into an array - e.g. 1, 2
						q = '';
						for (var i in pinnedEntryIds) {
							q += (q == "" ? "" : ", ") + "?";
						}
						var noRows = 0;
						var doneRows = 0;
						tx.executeSql('SELECT entry_id, order_number FROM budgets_entries WHERE entry_id IN (' + q + ') GROUP BY entry_id ORDER BY order_number', pinnedEntryIds,
							function(tx, results) {
								noRows = results.rows.length;
								for (var i=0; i < results.rows.length; i++) {
									
									// insert pinned entries and order numbers to budgets_entries with id of new budget id
									var entryId = results.rows.item(i).entry_id;
									// var orderNumber = results.rows.item(i).order_number;
									tx.executeSql('INSERT INTO budgets_entries(budget_id, entry_id, order_number) VALUES (?,?,?)', [budgetId, entryId, i+1], function() {
										doneRows++;
									}, balance.webdb.onError); // NOTE: changed orderNumber to i+1
									
								}
								
							}, null,
						balance.webdb.onError);

						// wait til all rows have been updated before saving database to dropbox
						var checkDone = setInterval(function(){
							if (doneRows === noRows) {
								clearInterval(checkDone);
								localChanges = true;
								balance.webdb.getAllEntries(budgetId, loadAllEntries);
							}
						}, 100);
						
					},
					balance.webdb.onError);

			});
		} 

	/*
		> get all entries
	*/
	
		balance.webdb.getAllEntries = function(id, callback) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				// check if budget exists
				tx.executeSql('SELECT budgets.id FROM budgets WHERE budgets.id = ?', [id], function(tx, rs) {
					// if budget exists
					if (rs.rows.length > 0) {
						// get entry id's from budgets_entries
						tx.executeSql('SELECT entry_id FROM budgets_entries WHERE budget_id=?', [id], function(tx, rs) {
							var entryIds = new Array();
							for (var i=0; i < rs.rows.length; i++) {
								entryIds.push(rs.rows.item(i).entry_id);
							}
							// get entries from entries
							q = '';
							for (var i in entryIds) {
								q += (q == "" ? "" : ", ") + "?";
							}
							tx.executeSql('SELECT DISTINCT entries.* FROM entries JOIN budgets_entries on entries.id = budgets_entries.entry_id WHERE entries.id IN (' + q + ') AND budgets_entries.budget_id = ' + id + ' ORDER BY budgets_entries.order_number', entryIds, callback, balance.webdb.onError);
						}, balance.webdb.onError);
					}
					// else budget doesn't exist
					else {
						// go back to budgets
						$('.back-to-budgets').trigger('click');
					}
				}, balance.webdb.onError)


			});
		}
		
	/*
		> add entry
	*/
	
		balance.webdb.addEntry = function(budgetId, orderNumber, isExpense, $entryMarkup) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				var title = '';
				var dateCreated = new Date();
				// add to entries
				tx.executeSql('INSERT INTO entries(title, isexpense, date_created) VALUES (?,?,?)', [title, isExpense, dateCreated], 
					function(tx, results) {
						entryId = results.insertId;
						// add to budgets_entries
						tx.executeSql('INSERT INTO budgets_entries(budget_id, entry_id, order_number) VALUES (?,?,?)', [budgetId, entryId, orderNumber], 
							function(tx, results) {
								addIdToEntryMarkup($entryMarkup, entryId);
								localChanges = true;
							},
							balance.webdb.onError);
					},
					balance.webdb.onError);
			});
		}
		
	/*
		> edit entry
	*/
	
		balance.webdb.editEntry = function(id, title, amount, pinned, isexpense) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {				
				tx.executeSql("UPDATE entries SET title = ?, amount = ?, pinned = ?, isexpense = ? WHERE id = ?", [title, amount, pinned, isexpense, id], function() {
					localChanges = true;
				}, balance.webdb.onError);
			});
		}
	
		
	/*
		> update entry order numbers
	*/
	
		balance.webdb.updateEntryOrderNumbers = function(entryIds, orderNumbers, budgetIds) {
			var noRows = entryIds.length;
			var doneRows = 0;
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				for (i=0; i<entryIds.length; i++) {
					tx.executeSql("UPDATE budgets_entries SET order_number = ? WHERE entry_id = ? AND budget_id = ?", [orderNumbers[i], entryIds[i], budgetIds[i]], function(tx, rs) {
						doneRows++;
					}, balance.webdb.onError);
				}
			});
			// wait til all rows have been updated before saving database to dropbox
			var checkDone = setInterval(function(){
				if (doneRows === noRows) {
					clearInterval(checkDone);
					localChanges = true;
				}
			}, 100);
		}
				
		
	/*
		> delete entry
	*/
	
		balance.webdb.deleteEntry = function(entryId, budgetId) {
			var db = balance.webdb.db;
			var numberOfEntries = 0;
			db.transaction(function(tx){
				tx.executeSql('SELECT COUNT(*) AS mycount FROM budgets_entries WHERE entry_id=?', [entryId],
					function(tx, rs) {
						numberOfEntries = rs.rows.item(0).mycount; // if this count is only 1, we should delete the entry from 'entries' too
						tx.executeSql('DELETE FROM budgets_entries WHERE entry_id=? AND budget_id=? ', [entryId, budgetId],
							function(tx, rs) {
								if (numberOfEntries == 1) {
									tx.executeSql('DELETE FROM entries WHERE ID=?', [entryId], function() {
										localChanges = true;
									}, balance.webdb.onError);
								}
							},
						balance.webdb.onError);
					},
				balance.webdb.onError);
			});	
		}
		
		
	/*
		> pin entry
	*/
	
		balance.webdb.pinEntry = function(entryId) {
			var db = balance.webdb.db;
			db.transaction(function(tx){
				tx.executeSql("UPDATE entries SET pinned = ? WHERE id = ?", [1, entryId], function() {
					localChanges = true;
				}, balance.webdb.onError);
			});	
		}
		
		
	/*
		> unpin entry
	*/
	
		// make all pinned entries that get unpinned into unique entries
		balance.webdb.unpinEntry = function(unpinnedEntryId, currentlyViewedBudgetId) {
			var db = balance.webdb.db;
			// get entry id, title and amount of unpinned entry and set pinned to 0 in entries
			var unpinnedEntryTitle;
			var unpinnedEntryAmount;
			var isexpense;
			db.transaction(function(tx){
				tx.executeSql("SELECT amount, title, isexpense FROM entries WHERE id = ?", [unpinnedEntryId], function(tx, rs) {
					unpinnedEntryTitle = rs.rows.item(0).title;
					unpinnedEntryAmount = rs.rows.item(0).amount;
					isexpense = rs.rows.item(0).isexpense;
					tx.executeSql("UPDATE entries SET pinned = ? WHERE id = ?", [0, unpinnedEntryId], function(tx, rs) {
						// get id's of all entries in budgets_entries where entry_id is unpinned entry id (into array)
						budgetsEntriesIds = new Array;
						newIds = new Array;
						tx.executeSql('SELECT id FROM budgets_entries WHERE entry_id = ?', [unpinnedEntryId], 
							function(tx, rs) {
								// create array of entry id's (but not first one)
								for (i=0; i < rs.rows.length; i++) {
									if (i > 0) {
										budgetsEntriesIds.push(rs.rows.item(i).id);
									}
								}

								var noRows = budgetsEntriesIds.length;
								var doneRows = 0;

								// for every item in array 
								for (j = 0; j < budgetsEntriesIds.length; j++) {
									    
									// create a new entry in entries
									var dateCreated = new Date();
									tx.executeSql('INSERT INTO entries(title, amount, isexpense, date_created) VALUES(?,?,?,?)', [unpinnedEntryTitle, unpinnedEntryAmount, isexpense, dateCreated], null, balance.webdb.onError);
									
									// grab new id from entries
									tx.executeSql('SELECT id FROM entries ORDER BY id DESC LIMIT 1', [],
										function(tx, results) {
											newId = results.rows.item(0).id;
											newIds.push(newId);
											doneRows++;
										}
									, balance.webdb.onError)

								}
								
								// wait til all rows have been updated before saving database to dropbox
								var checkDone = setInterval(function(){
									if (doneRows === noRows) {
										clearInterval(checkDone);
										
										// update budgets_entries once the above is complete
										var noRows2 = newIds.length;
										var doneRows2 = 0;
										db.transaction(function(tx){
											for (k=0; k < newIds.length; k++) {
												id = newIds[k];
												entryId = budgetsEntriesIds[k];
												tx.executeSql('UPDATE budgets_entries SET entry_id = ? WHERE id = ?', [id, entryId], function() {
													doneRows2++;
												}, balance.webdb.onError)
											}
										});

										var checkDone2 = setInterval(function(){
											if (doneRows2 === noRows2) {
												clearInterval(checkDone2);
												console.log('all done');;
												localChanges = true;
												balance.webdb.getAllEntries(currentlyViewedBudgetId, loadAllEntries);
											}
										}, 100);

									}
								}, 100);

								
						}, balance.webdb.onError);
						 
					}, balance.webdb.onError);
				}, balance.webdb.onError);
			});
				
		}


	/*
		> mark as paid
	*/
	
		balance.webdb.markAsPaid = function(budgetId, entryId) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {

				// get budgets_entries id
				tx.executeSql('SELECT id FROM budgets_entries WHERE budget_id=? AND entry_id=?', [budgetId, entryId], function(tx, rs) {
					var budgetsEntriesId = rs.rows.item(0).id;

					// add to 'paid' table
					tx.executeSql('INSERT INTO paid(budgets_entries_id) VALUES (?)', [budgetsEntriesId], function() {
						localChanges = true;
					}, balance.webdb.onError);

				}, balance.webdb.onError);

			});
		}

	/*
		> mark as unpaid
	*/
	
		balance.webdb.markAsUnpaid = function(budgetId, entryId) {
			var db = balance.webdb.db;
			db.transaction(function(tx) {

				// get budgets_entries id
				tx.executeSql('SELECT id FROM budgets_entries WHERE budget_id=? AND entry_id=?', [budgetId, entryId], function(tx, rs) {
					var budgetsEntriesId = rs.rows.item(0).id;

					// remove from 'paid' table
					tx.executeSql('DELETE FROM paid WHERE budgets_entries_id=?', [budgetsEntriesId], function() {
						localChanges = true;
					}, balance.webdb.onError);

				}, balance.webdb.onError);

			});
		}

	/*
		> is paid - find out if an entry is paid
	*/

		balance.webdb.isPaid = function(budgetId, entryId) {
			// var isPaid = false;
			var dfrd = $.Deferred();
			var db = balance.webdb.db;
			db.transaction(function(tx) {

				// get budgets_entries id
				tx.executeSql('SELECT id FROM budgets_entries WHERE budget_id=? AND entry_id=?', [budgetId, entryId], function(tx, rs) {
					var budgetsEntriesId = rs.rows.item(0).id;

					// look for item in 'paid'
					tx.executeSql('SELECT * FROM paid WHERE budgets_entries_id=?', [budgetsEntriesId], function(tx, rs) {
						if (rs.rows.length > 0) {
							dfrd.resolve('true');
						}
						else {
							dfrd.resolve('false');
						}
					}, balance.webdb.onError);

				}, balance.webdb.onError);

			});

			return dfrd.promise();
		}

	/*
		> has starred entries - find out if user has some starred entries that are attached to an existing budget
	*/

		// grab id's of all pinned entries in 'entries'
		// check if any of these id's are present in budgets_entries in 'entry_id' column
		balance.webdb.hasStarredEntries = function(budgetId) {
			var dfrd = $.Deferred();
			var db = balance.webdb.db;
			db.transaction(function(tx) {
				// get all ids of pinned entries from 'entries' into an array - e.g. 1, 2
				var pinnedEntryIds = new Array;
				tx.executeSql('SELECT id FROM entries WHERE pinned = ?', [1], 
					function(tx, results) {
						for (var i=0; i < results.rows.length; i++) {
							pinnedEntryIds.push(results.rows.item(i).id);
						}
						// get order_number of each pinned entry based on previousBudgetId and 'entry_id' into an array - e.g. 1, 2
						q = '';
						for (var i in pinnedEntryIds) {
							q += (q == "" ? "" : ", ") + "?";
						}
						tx.executeSql('SELECT entry_id, order_number FROM budgets_entries WHERE entry_id IN (' + q + ') GROUP BY entry_id ORDER BY order_number', pinnedEntryIds,
							function(tx, results) {
								for (var i=0; i < results.rows.length; i++) {
									
									if (results.rows.length > 0) {
										dfrd.resolve('true');
									}
									else {
										dfrd.resolve('false');
									}
									
								}
								
							}, null,
							balance.webdb.onError);
						
					},
					balance.webdb.onError);

			});

			return dfrd.promise();
		} 
