/*
* tutorial.js
*
* CONTENTS
*
* tutorial carousel
* tutorial animations
*    animation 1
*    animation 2
*    animation 3
*    animation 4
*    animation 5
*    animation 6
*/

/*
	> tutorial carousel
*/

	disableTutorialSwiping = false;

	var currentNav = 1;

    function Carousel(element)
    {
        var self = this;
        element = $(element);

        var container = $(">.tutorial-slides", element);
        var panes = $(">.tutorial-slides>div", element);

        var pane_width = 0;
        var pane_count = panes.length;

        var current_pane = 0;


        /**
         * initial
         */
        this.init = function() {
            setPaneDimensions();

            $(window).on("load resize orientationchange", function() {
                setPaneDimensions();
            })
        };


        /**
         * set the pane dimensions and scale the container
         */
        function setPaneDimensions() {
            pane_width = element.width();
            panes.each(function() {
                $(this).width(pane_width);
            });
            container.width(pane_width*pane_count);
        };


        /**
         * show pane by index
         */
        
        var previousNav = 0;
         
        this.showPane = function(index, animate, direction) {
        	
            // between the bounds
            index = Math.max(0, Math.min(index, pane_count-1));
            current_pane = index;

            var offset = -((100/pane_count)*current_pane);
            setContainerOffset(offset, animate);
            
            // set active tutorial-nav
            currentNav = current_pane + 1;
            $('.tutorial-nav a').removeClass('active');
            $('.tutorial-nav a:nth-child(' + currentNav + ')').addClass('active');
            
            // stop the previous tutorial animation start current tutorial animation            
            // only do stuff if we're on a different slide from before
            if (currentNav != previousNav) {
	        	// disable swiping for a second
	        	disableTutorialSwiping = true;
	        	setTimeout(function(){
	        		disableTutorialSwiping = false;
	        	}, 1000);
	        	
            	// if user swiped forwards
	            if (direction == 'forwards') {
	            	// stop previous slide's animation
	            	stopTutorialAnimation[currentNav-1] = true;
	            	setTimeout(function(){
			            tutorialAnimation(currentNav-1, 'reset');
	            	}, 100);

	            	// start current slide's animation
            		stopTutorialAnimation[currentNav] = false;
			        tutorialAnimation(currentNav);

	            }
            	// if user swiped backwards
	            else if (direction == 'backwards') {
	            	// stop previous slide's animation
	            	stopTutorialAnimation[currentNav+1] = true;
	            	setTimeout(function(){
			            tutorialAnimation(currentNav+1, 'reset');          		
	            	}, 100);

	            	// start current slide's animation
            		stopTutorialAnimation[currentNav] = false;
			        tutorialAnimation(currentNav);

	            }
            }
            
            
            previousNav = currentNav;
        };


        function setContainerOffset(percent, animate) {
            container.removeClass("animate");

            if(animate) {
                container.addClass("animate");
            }

           container.css("transform", "translate("+ percent +"%,0)");
        }

        this.next = function() { return this.showPane(current_pane+1, animateTutorialSlides, 'forwards'); };
        this.prev = function() { return this.showPane(current_pane-1, animateTutorialSlides, 'backwards'); };


        /**
         * handle next / previous taps
         */

        $('.tutorial-next').click(function (e) {
        	e.preventDefault();
        	self.next();
        });
        $('.tutorial-prev').click(function (e) {
        	e.preventDefault();
        	self.prev();
        });



        function handleHammer(ev) {
            // disable browser scrolling
            ev.gesture.preventDefault();

            switch(ev.type) {
                case 'dragright':
                case 'dragleft':
                	// don't allow dragging for a short time if slide just changed
                	if (disableTutorialSwiping == true) {
                		break;
                	}
                
                    // stick to the finger
                    var pane_offset = -(100/pane_count)*current_pane;
                    var drag_offset = ((100/pane_width)*ev.gesture.deltaX) / pane_count;

                    // slow down at the first and last pane
                    if((current_pane == 0 && ev.gesture.direction == "right") ||
                        (current_pane == pane_count-1 && ev.gesture.direction == "left")) {
                        drag_offset *= .4;
                    }

                    setContainerOffset(drag_offset + pane_offset);
                    break;

                case 'swipeleft':
	                // don't allow dragging for a short time if slide just changed
	                if (disableTutorialSwiping == true) {
	                	break;
	                }
                    self.next();
                    ev.gesture.stopDetect();
                    break;

                case 'swiperight':
                	// don't allow dragging for a short time if slide just changed
                	if (disableTutorialSwiping == true) {
                		break;
                	}
                    self.prev();
                    ev.gesture.stopDetect();
                    break;

                case 'release':
                	// don't allow dragging for a short time if slide just changed
                	if (disableTutorialSwiping == true) {
                		break;
                	}
                    // more then 50% moved, navigate
                    if(Math.abs(ev.gesture.deltaX) > pane_width/2) {
                        if(ev.gesture.direction == 'right') {
                            self.prev();
                        } else {
                            self.next();
                        }
                    }
                    else {
                        self.showPane(current_pane, true, 'none');
                    }
                    break;
            }
        }

        new Hammer(element[0], { dragLockToAxis: true }).on("release dragleft dragright swipeleft swiperight", handleHammer);
    }
		

/* 
	> tutorial animations
*/

	var numberOfTutorialAnimations = $('.tutorial-slide').length;
	var stopTutorialAnimation = new Array;

	for (var i = 1; i < numberOfTutorialAnimations + 1; i++) { 
		// create a stop variable for each animation
		stopTutorialAnimation[i] = false;
	}	
	
	/*$('#tutorial').css({y: 0, display: 'block'});
	carousel.init();*/

	var carousel = new Carousel(".tutorial-slides-container");

	// show on first run
	if (!localStorage.firstRunTutorialDisplayed) {
		$('#tutorial').show().css( {y: '0%'} );
		carousel.init();
	}	
			
	// tutorialAnimation
	function tutorialAnimation(id, reset) {
		
		switch(id) {
			
			/*
				>> animation 1
			*/
			
				case 1:
				
					// no animation for first slide
				
					break;
			
			/* 
				>> animation 2
			*/
			
				case 2:
								
					var $touchIndicator2 = $('.slide-2 .touch-indicator');
					var $tutorialBudget2 = $('.slide-2 .item');
					var $tutorialBudgetTitle2 = $('.slide-2 .item-title');
					var $tutorialBudgetDetails2 = $('.slide-2 .budgets .item:first-child .item-details');
					
					// start animation
					if (reset != 'reset') {
						// move indicator to 'Add Budget'
						if (!stopTutorialAnimation[id]) {
							// reset title again in case timers still running     x / 278
							$tutorialBudgetTitle2.stop(true, true).text('');
							$touchIndicator2.delay(500).animate({
								left: '46%', top: '50px'
							}, 300, function() {
								// tap indicator
								if (!stopTutorialAnimation[id]) {
									$touchIndicator2.transition({
										scale: '0.9',
										opacity: 0.6
									}, 100, function() {
										// untap indicator
										if (!stopTutorialAnimation[id]) {
											$touchIndicator2.transition({
												scale: '1',
												opacity: 0.3
											}, 100, function() {
												// create budget
												if (!stopTutorialAnimation[id]) {
													$tutorialBudgetTitle2.text('');
													$tutorialBudget2.animate({
														height: 49
													}, 500, function() {
														// type August
														if (!stopTutorialAnimation[id]) {
															setTimeout(function(){
																$tutorialBudgetTitle2.text('A|');
																if (!stopTutorialAnimation[id]) {
																	setTimeout(function(){
																		$tutorialBudgetTitle2.text('Au');
																		if (!stopTutorialAnimation[id]) {
																			setTimeout(function(){
																				$tutorialBudgetTitle2.text('Aug|');
																				if (!stopTutorialAnimation[id]) {
																					setTimeout(function(){
																						$tutorialBudgetTitle2.text('Augu');
																						if (!stopTutorialAnimation[id]) {
																							setTimeout(function(){
																								$tutorialBudgetTitle2.text('Augus|');
																								if (!stopTutorialAnimation[id]) {
																									setTimeout(function(){
																										$tutorialBudgetTitle2.text('August');
																										if (!stopTutorialAnimation[id]) {
																											setTimeout(function(){
																												$tutorialBudgetTitle2.text('August|');
																												if (!stopTutorialAnimation[id]) {
																													setTimeout(function(){
																														$tutorialBudgetTitle2.text('August');
																														// move indicator to top item
																														if (!stopTutorialAnimation[id]) {
																															$touchIndicator2.delay(500).animate({
																																top: '50px',
																																left: '78%'
																															}, 300, function() {
																																
																																// tap indicator
																																if (!stopTutorialAnimation[id]) {
																																	$touchIndicator2.delay(300).transition({
																																		scale: '0.9',
																																		opacity: 0.6		
																																	}, 100, function() {
																																		// move indicator across to the left to indicate left swipe
																																		if (!stopTutorialAnimation[id]) {
																																			$touchIndicator2.delay(200).animate({
																																				left: '1%'
																																			}, 300, function() {
																																				// untap indicator
																																				if (!stopTutorialAnimation[id]) {
																																					$touchIndicator2.transition({
																																						scale: '1',
																																						opacity: 0.3		
																																					}, 100);
																																				}
																																			});	
																																		}
																																		if (!stopTutorialAnimation[id]) {
																																			// swipe top item and show options
																																			$tutorialBudgetDetails2.delay(500).transition({
																																				x: '-165'
																																			}, 200, function() {
																																				// move indicator to delete button
																																				if (!stopTutorialAnimation[id]) {
																																					$touchIndicator2.delay(600).animate({
																																						left: '100%',
																																						marginLeft: '-55px'
																																					}, 300, function() {
																																						// tap indicator
																																						if (!stopTutorialAnimation[id]) {
																																							$touchIndicator2.delay(500).transition({
																																								scale: '0.9',
																																								opacity: 0.6
																																							}, 100, function() {
																																								// untap indicator
																																								if (!stopTutorialAnimation[id]) {
																																									$touchIndicator2.transition({
																																										scale: '1',
																																										opacity: 0.3		
																																									}, 100);
																																								}
																																								// delete budget
																																								if (!stopTutorialAnimation[id]) {
																																									$tutorialBudget2.animate({
																																										height: '0'
																																									}, 100, function() {
																																										// reset elements
																																										if (!stopTutorialAnimation[id]) {
																																											setTimeout(function(){
																																												tutorialAnimation(id, 'reset');
																																												tutorialAnimation(id);
																																											}, 2000);
																																										}
																																									});
																																								}
																																								
																																							});		
																																						}
																																					});													
																																				}
																																			});
																																		}
																																	});
																																}
																																
																																
																																
																															});
																														}
																													}, 200);
																												}
																											}, 200);
																										}
																									}, 200);
																								}	
																							}, 200);
																						}
																					}, 200);
																				}	
																			}, 200);
																		}
																	}, 200);
																}
															}, 200);
														}
													});
												}
											});
										}
									});
								}
							});
						}
					}
					// stop and reset animation if 'reset' was specified
					else {
						$touchIndicator2.stop(true, true).css({left: '65%', top: '142px', opacity: 0.3, marginLeft: 0}).show();
						$tutorialBudgetTitle2.stop(true, true).text('');
						$tutorialBudget2.stop(true, true).css({height: 0});
						$tutorialBudgetDetails2.stop(true,true).css({x: 0});
					}
				
					break;	


			/* 
				>> animation 3
			*/
			
				case 3:
				
					var $touchIndicator3 = $('.slide-3 .touch-indicator');
					var $tutorialBudget3 = $('.slide-3 .budgets .item:first-child');
					var $entriesContainer3 = $('.slide-3 .entries-container');
					var $entry3 = $('.slide-3 .entries .item:first-child');
					var $entryTitle3 = $('.slide-3 .entries .item:first-child p.entry-title');
					var $entryValue3 = $('.slide-3 .entries .item:first-child p.amount');
					var $info3 = $('.slide-3 .info');
					
					// start animation
					if (reset != 'reset') {
						// move indicator to top item
						if (!stopTutorialAnimation[id]) {
							$touchIndicator3.delay(500).animate({
								top: '50px'
							}, 300, function() {
								// tap indicator
								if (!stopTutorialAnimation[id]) {
									$touchIndicator3.delay(500).transition({
										scale: '0.9',
										opacity: 0.6
									}, 100, function() {
										// add touch state to top budget
										if (!stopTutorialAnimation[id]) {
											$tutorialBudget3.addClass('touch');
											// untap indicator
											$touchIndicator3.transition({scale: '1', opacity: 0.3});
											// slide in entries
											$entriesContainer3.show().delay(500).transition({
												x: '0%'
											}, 200, function() {
												// move touch indicator over 'Add Income'
												if (!stopTutorialAnimation[id]) {
													$touchIndicator3.delay(400).animate({
														left: '18%'
													}, 500, function() {
														// tap indicator
														if (!stopTutorialAnimation[id]) {
															$touchIndicator3.delay(500).transition({
																scale: '0.9',
																opacity: 0.6
															}, 100, function() {
																// untap indicator
																if (!stopTutorialAnimation[id]) {
																	$touchIndicator3.transition({scale: '1', opacity: 0.3});
																}
																// create entry
																if (!stopTutorialAnimation[id]) {
																	$entry3.animate({
																		height: 49
																	}, 400, function() {
																		// tap indicator
																		if (!stopTutorialAnimation[id]) {
																			$touchIndicator3.delay(750).transition({
																				scale: '0.9',
																				opacity: 0.6
																			}, 100, function() {
																				// untap indicator
																				if (!stopTutorialAnimation[id]) {
																					$touchIndicator3.transition({
																						scale: '1', 
																						opacity: 0.3
																					}, 100);
																				}
																				if (!stopTutorialAnimation[id]) {
																					// reset titles just in case timers are still running
																					$entryTitle3.css('color', '#B1D2C7').text('E.g. Salary');
																					$entryValue3.text('+ $0.00');
																					setTimeout(function(){
																						$entryTitle3.text('|').removeAttr('style');
																						if (!stopTutorialAnimation[id]) {
																							setTimeout(function(){
																								$entryTitle3.text('S');
																								if (!stopTutorialAnimation[id]) {
																									setTimeout(function(){
																										$entryTitle3.text('Sa|');	
																										if (!stopTutorialAnimation[id]) {
																											setTimeout(function(){
																												$entryTitle3.text('Sal');
																												if (!stopTutorialAnimation[id]) {
																													setTimeout(function(){
																														$entryTitle3.text('Sala|');
																														if (!stopTutorialAnimation[id]) {
																															setTimeout(function(){
																																$entryTitle3.text('Salar');
																																if (!stopTutorialAnimation[id]) {
																																	setTimeout(function(){
																																		$entryTitle3.text('Salary');
																																		// move indicator to entry amount
																																		if (!stopTutorialAnimation[id]) {
																																			$touchIndicator3.delay(500).animate({
																																				left: '100%',
																																				marginLeft: '-78px',
																																				top: 50
																																			}, 500, function() {
																																				// tap indicator
																																				if (!stopTutorialAnimation[id]) {
																																					$touchIndicator3.delay(500).transition({
																																						scale: '0.9',
																																						opacity: 0.6
																																					}, 100, function() {
																																						// fade out touch indicator
																																						if (!stopTutorialAnimation[id]) {
																																							$touchIndicator3.fadeOut();
																																							$entryValue3.text('|');
																																							if (!stopTutorialAnimation[id]) {
																																								setTimeout(function(){
																																									$entryValue3.text('2|');
																																									if (!stopTutorialAnimation[id]) {
																																										setTimeout(function(){
																																											$entryValue3.text('20|');
																																											if (!stopTutorialAnimation[id]) {
																																												setTimeout(function(){
																																													$entryValue3.text('200|');
																																													if (!stopTutorialAnimation[id]) {
																																														setTimeout(function(){
																																															$entryValue3.text('2000|');
																																															if (!stopTutorialAnimation[id]) {
																																																setTimeout(function(){
																																																	$entryValue3.text('2000.|');
																																																	if (!stopTutorialAnimation[id]) {
																																																		setTimeout(function(){
																																																			$entryValue3.text('2000.3|');
																																																			if (!stopTutorialAnimation[id]) {
																																																				setTimeout(function(){
																																																					$entryValue3.text('2000.34');
																																																					if (!stopTutorialAnimation[id]) {
																																																						setTimeout(function(){
																																																							$entryValue3.text('+ $2,000.34');
																																																							// show info
																																																							if (!stopTutorialAnimation[id]) {
																																																								$info3.fadeIn();
																																																								// reset elements
																																																								if (!stopTutorialAnimation[id]) {
																																																									setTimeout(function(){
																																																										tutorialAnimation(id, 'reset');
																																																										tutorialAnimation(id);
																																																									}, 2000);
																																																								}
																																																							}
																																																						}, 1000);
																																																					}
																																																				}, 200);
																																																			}
																																																		}, 200);
																																																	}
																																																}, 200);
																																															}
																																														}, 200);
																																													}
																																												}, 200);
																																											}
																																										}, 200);
																																									}
																																								}, 200);
																																							}
																																						}
																																									
																																					});					
																																				}
																																			});	
																																				
																																		}	
																																	}, 200);
																																}		
																															}, 200);
																														}		
																													}, 200);
																												}	
																											}, 200);
																										}		
																									}, 200);
																								}	
																							}, 200);
																						}
																					}, 200);
																				}	

																			});
																		}
																	});
																}
															});
														}
														
													});	
												}
											});
										}
									});
								}
							});
						}							
					}
					
					// stop and reset animation if 'reset' was specified
					else {
						$touchIndicator3.stop(true, true).transition({scale: 1}).css({left: '65%', top: 177, display: 'block', opacity: 0.3, marginLeft: 0});
						$tutorialBudget3.stop(true, true).removeClass('touch');
						$entriesContainer3.stop(true, true).css({x: '100%'});
						$entry3.stop(true, true).css({height: 0});
						$entryTitle3.css('color', '#B1D2C7').text('E.g. Salary');
						$entryValue3.text('+ $0.00');
						$info3.stop(true, true).hide();
					}
				
					break;	


			/* 
				>> animation 4
			*/
			
				case 4:
				
					var $touchIndicator4 = $('.slide-4 .touch-indicator');
					var $entry4 = $('.slide-4 .entries .item:first-child');
					var $entryDetails4 = $('.slide-4 .entries .item:first-child .entry-details');
					var $star4 = $('.slide-4 .entries .item:first-child .pin, .slide-4 .entries .item:first-child .unpin');
					var $starIcon = $('.slide-4 .entries .item:first-child .pin i, .slide-4 .entries .item:first-child .unpin i')
					
					// start animation
					if (reset != 'reset') {
						// move indicator to top item
						if (!stopTutorialAnimation[id]) {
							$touchIndicator4.delay(500).animate({
								top: '50px'
							}, 300, function() {
									
								// tap indicator
								if (!stopTutorialAnimation[id]) {
									$touchIndicator4.delay(500).transition({
										scale: '0.9',
										opacity: 0.6
									}, 100, function() {
										// move indicator across to the left to indicate left swipe
										if (!stopTutorialAnimation[id]) {
											$touchIndicator4.delay(200).animate({
												left: '1%'
											}, 300, function() {
												// untap indicator
												if (!stopTutorialAnimation[id]) {
													$touchIndicator4.transition({
														scale: '1',
														opacity: 0.3
													}, 100);
												}
											});	
										}
										if (!stopTutorialAnimation[id]) {
											// swipe top item and show options
											$entryDetails4.delay(500).transition({
												x: '-165'
											}, 200, function() {
												// move indicator to star button
												if (!stopTutorialAnimation[id]) {
													$touchIndicator4.delay(1000).animate({
														left: '100%',
														marginLeft: '-100px'
													}, 300, function() {
														// tap indicator
														if (!stopTutorialAnimation[id]) {
															$touchIndicator4.delay(500).transition({
																scale: '0.9',
																opacity: 0.6
															}, 100, function() {
																// star entry
																if (!stopTutorialAnimation[id]) {
																	$touchIndicator4.fadeOut();
																	$entry4.addClass('pinned');
																	$star4.addClass('unpin').removeClass('pin');
																	$starIcon.addClass('icon-star-filled').removeClass('icon-star');
																	// slide entry details back across
																	if (!stopTutorialAnimation[id]) {
																		$entryDetails4.delay(500).transition({
																			x: '0'
																		}, 200, function() {
																			// reset elements
																			if (!stopTutorialAnimation[id]) {
																				setTimeout(function(){
																					tutorialAnimation(id, 'reset');
																					tutorialAnimation(id);
																				}, 2000);
																			}																		
																		});
																	}
																}
															});		
														}
													});													
												}
											});
										}
									});
								}
								
							});
						}						
					}
					
					// stop and reset animation if 'reset' was specified
					else {
						$touchIndicator4.stop(true, true).transition({scale: 1}).css({left: '78%', top: 180, display: 'block', opacity: 0.3, marginLeft: 0});
						$entry4.stop(true, true).removeClass('pinned');
						$entryDetails4.stop(true, true).css({x: '0'});
						$star4.removeClass('unpin').addClass('pin');
						$starIcon.removeClass('icon-star-filled').addClass('icon-star');
					}
				
					break;	

			/* 
				>> animation 5
			*/
			
				case 5:
				
					var $touchIndicator5 = $('.slide-5 .touch-indicator');
					var $entry5 = $('.slide-5 .entries .item:last-child');
					var $entryFirst5 = $('.slide-5 .entries .item:first-child');
					var $placeholder5 = $('<div class="placeholder" style="height: 49px"></div>');
					
					// start animation
					if (reset != 'reset') {
						// move indicator to top item
						if (!stopTutorialAnimation[id]) {
							$touchIndicator5.delay(1000).animate({
								left: '48%',
								top: 145
							}, 300, function() {
								// tap indicator
								if (!stopTutorialAnimation[id]) {
									$touchIndicator5.delay(500).transition({
										scale: '0.9',
										opacity: 0.6
									}, 100, function() {
										// start dragging
										if (!stopTutorialAnimation[id]) {
											// get original y offset of entry
											originalOffset = $entry5.position().top;
											// add dragging class and pull out of the page
											$entry5.delay(500).addClass('dragging').transit({scale: 1.05}).css('top', originalOffset + 'px').after($placeholder5);;
											// move touch indicator and drag entry upwards, above middle item
											if (!stopTutorialAnimation[id]) {
												setTimeout(function(){
													if (!stopTutorialAnimation[id]) {
														$touchIndicator5.animate({
															top: 87
														}, { duration: 500, queue: false });
													}														
													if (!stopTutorialAnimation[id]) {
														$entry5.animate({
															top: 87
														}, 500, function() {
															// move placeholder and drop item
															if (!stopTutorialAnimation[id]) {
																$placeholder5.insertAfter($entryFirst5);
																setTimeout(function(){
																	// scale back down
																	if (!stopTutorialAnimation[id]) {
																		$touchIndicator5.transition({scale: '1', opacity: 0.3}, 100);
																		$entry5.transit({
																			scale: 1,
																			top: 100
																		}, 200, function() {
																			$entry5.removeClass('dragging');
																			$entry5.insertAfter($placeholder5)
																			$entry5.removeAttr('style');
																			$placeholder5.remove();
																			// reset elements
																			if (!stopTutorialAnimation[id]) {
																				setTimeout(function(){
																					tutorialAnimation(id, 'reset');
																					tutorialAnimation(id);
																				}, 2000);
																			}
																		});
																	}
																}, 1000);
															}
														});
													}
												}, 1000);
											}
											
										}
									});
								}
															
							});
						}						
					}
					
					// stop and reset animation if 'reset' was specified
					else {
						$touchIndicator5.stop(true, true).transition({scale: 1}).css({left: '78%', top: 180, display: 'block', opacity: 0.3});
						$('.slide-5 .entries .entry').stop(true, true).css({scale: 1, top: 0}).removeAttr('style').removeClass('dragging');
						$('.slide-5 .entries .placeholder').remove();
					}
				
					break;	


			/* 
				>> animation 6
			*/
			
				case 6:
				
					var $touchIndicator6 = $('.slide-6 .touch-indicator');
					var $entriesContainer6 = $('.slide-6 .entries-container');
					var $budgetsContainer6 = $('.slide-6 .budgets-container');
					
					// start animation
					if (reset != 'reset') {
						// move indicator to top item
						if (!stopTutorialAnimation[id]) {
							$touchIndicator6.delay(1000).animate({
								left: '5%',
								top: 103
							}, 300, function() {
								// tap indicator
								if (!stopTutorialAnimation[id]) {
									$touchIndicator6.delay(500).transition({
										scale: 0.9,
										opacity: 0.6
									}, 200, function() {
										// swipe to the right
										if (!stopTutorialAnimation[id]) {
											// swipe indicator
											$touchIndicator6.animate({
												left: '77%'
											}, 300);
											// swipe entries
											$entriesContainer6.delay(200).transition({
												x: '100%'
											}, 300, function() {
												// untap indicator
												if (!stopTutorialAnimation[id]) {
													$touchIndicator6.transition({
														scale: 1,
														opacity: 0.3
													}, 200, function() {
														// move indicator back to the left
														if (!stopTutorialAnimation[id]) {
															$touchIndicator6.delay(300).animate({
																left: '5%'
															}, 300, function() {
																// tap indicator
																if (!stopTutorialAnimation[id]) {
																	$touchIndicator6.delay(500).transition({
																		scale: 0.9,
																		opacity: 0.6
																	}, 200, function() {
																		// swipe to the right
																		if (!stopTutorialAnimation[id]) {
																			// swipe indicator
																			$touchIndicator6.animate({
																				left: '77%'
																			}, 300);
																			// swipe budgets
																			$budgetsContainer6.delay(200).transition({
																				x: '100%'
																			}, 300, function() {
																				// untap indicator
																				if (!stopTutorialAnimation[id]) {
																					$touchIndicator6.transition({
																						scale: 1,
																						opacity: 0.3
																					}, 200, function() {
																						// reset elements
																						if (!stopTutorialAnimation[id]) {
																							setTimeout(function(){
																								tutorialAnimation(id, 'reset');
																								tutorialAnimation(id);
																							}, 3000);
																						}
																					});
																				}
																			});
																		}																
																	});
																}			
															});
														}
													});
												}
											});
											
										}					
									});	
								}
							});
						}						
					}
					
					// stop and reset animation if 'reset' was specified
					else {
						$touchIndicator6.stop(true, true).transition({scale: 1}).css({left: '78%', top: 180, opacity: 0.3});
						$entriesContainer6.stop(true, true).css({x: '0%'})
						$budgetsContainer6.stop(true, true).css({x: '0%'})
					}
				
					break;	

			
		}
		
		

	}
	// end tutorialAnimation
	
